/*
 * This file is (C) Chris Wohlgemuth 1999
 * It is part of the Audio/Date-CD-Creator package
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "audiofolder.hh"
#include "audiofolderhelp.h"
#include <stdio.h>
#include <stdlib.h>
#include "cddb.h"

extern PVOID pvAudioSharedMem;

extern BOOL GrabberSetupDone;

extern char chrMpg123Path[CCHMAXPATH];
extern BOOL bMpg123SwabBytes;

extern int iNumCD;
extern char cFirstCD;
extern char chosenCD[3];
extern char chrGrabberPath[CCHMAXPATH];
extern char chrGrabberOptions[CCHMAXPATH];
extern int bTrackNumbers;
extern BOOL bGrabberFirst;
extern char chrInstallDir[CCHMAXPATH];
extern char chrCDRecord[CCHMAXPATH];/* Path to cdrecord */
extern char chrDataCDROptions[CCHMAXPATH];
extern char chrAudioCDROptions[CCHMAXPATH];
extern LONG lCDROptions;
extern char chosenWriter[3];

extern char chrCdrdaoPath[CCHMAXPATH];
extern char chrCdrdaoDriver[100];
extern char chrDeviceName[CCHMAXPATH];
extern char chrWriterName[CCHMAXPATH];

extern int iBus;
extern int iTarget;
extern int iLun;
extern int iSpeed;
extern int iFifo;

extern BOOL setupDone;
extern ATOM atomUpdateStatusbar;

extern HMODULE hAudioResource;

/* Extern */
HMODULE queryModuleHandle(void);
HFILE openDrive(char * drive);
void closeDrive(HFILE hfDrive);
int CDQueryAudioCDTracks(HFILE hfDrive);
LONG CDDBDiscID(char * drive, CDDBINFO * cddbInfo);
void getTrackName(LONG lDiskID,int iNumTrack,char * text);
void sendConfig();
ULONG launchPMWrapper(PSZ parameter, PSZ folderPath, PSZ wrapperExe, PSZ pszTitle);
void getTrackName(LONG lDiskID,int iNumTrack,char * text);
ULONG messageBox( char* text, ULONG ulTextID , LONG lSizeText,
                  char* title, ULONG ulTitleID, LONG lSizeTitle,
                  HMODULE hResource, HWND hwnd, ULONG ulFlags);
void getMessage(char* text,ULONG ulID, LONG lSizeText, HMODULE hResource,HWND hwnd);
void _Optlink playTimeThreadFunc(void *arg);
void _Optlink scanbusThreadFunc(void *arg);
void _Optlink writeThreadFunc (void *arg);
PSZ buildAudioWriteParam(CWAudioFolder* thisPtr, PSZ trackname);
PSZ buildAudioWriteParamDAO(CWAudioFolder* thisPtr);
BOOL buildTocFile( CWAudioFolder *thisPtr, HWND hwndListBox);
ULONG launchWrapper(PSZ parameter, PSZ folderPath,HWND hwnd, PSZ wrapperExe, PSZ title="CDRecord/2");

/* Local */
MRESULT EXPENTRY trackDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY statusLineDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY selectDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY grabDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY writeStatusDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY dialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY trackDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY audioCdrecordOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY generalOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY grabberOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY audioCdrdaoOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2); 

void getTrackName(LONG lDiskID,int iNumTrack,char * text)
{
  HINI hini;
  ULONG keyLength;
  char profileName[CCHMAXPATH];
  char discKey[20];
  char trackKey[20];
  char title[100];
  char text2[100];

  /* Build full path for cddb.ini file */
  sprintf(profileName,"%s\\cddb.ini",chrInstallDir);       
  /* Open ini-file */
  hini=PrfOpenProfile(WinQueryAnchorBlock(HWND_DESKTOP),profileName);
  do{
    if(!hini) {
      /* Text:   "Warning! Cannot open CDDB.ini file!"
         Title:  "Audio-CD-Creator"
         */
      messageBox( text2, IDSTR_NOTRACKNAMETEXT , sizeof(text2),
                  title, IDSTR_NOTRACKSSELECTEDTITLE, sizeof(title),
                  hAudioResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
      break;
    }/* end of if(!hini) */

    sprintf(discKey,"%X",lDiskID);
    sprintf(trackKey,"TRACK%d",iNumTrack);
    keyLength=PrfQueryProfileString(hini,discKey,trackKey,"",text,CCHMAXPATH);
    if(keyLength==1){
      break;/*  We have no trackname */
    }

    if(hini)PrfCloseProfile(hini);
    return;
  }while(TRUE);
  if(hini)PrfCloseProfile(hini);
  /* Text2: "Track %d" */
  getMessage(text2, IDSTR_LBTRACKNAME,sizeof(text2), hAudioResource,HWND_DESKTOP);

  sprintf(text, text2, iNumTrack);
}

/********************************************************************************/
/*                                                                              */
/* This procedure handles the statusline-dialog (frame extension) of the folder */
/*                                                                              */
/********************************************************************************/
MRESULT EXPENTRY statusLineDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  CWAudioFolder *thisPtr;
  HWND hwndMenu;
  POINTL ptl;
  SHORT a,b;
    
  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(hwnd,QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      return (MRESULT) TRUE;
    case WM_HELP:
      break;
    case WM_DESTROY:
      /* The dialog closes and gets destroyed */     
      break;	
    case WM_BUTTON2DOWN:
      /* Display popup menu for status bar */
      WinQueryPointerPos(HWND_DESKTOP,&ptl);
      WinMapWindowPoints(HWND_DESKTOP,hwnd,&ptl,1);
      hwndMenu=WinWindowFromPoint(hwnd,&ptl,FALSE);/* Get child window under ptr */
      switch(WinQueryWindowUShort(hwndMenu,QWS_ID))/* Get child window id */
        {
        default:
          a=IDM_STATUSDEFAULTHELP;
          b=a;
          break;
        }
      hwndMenu=WinLoadMenu(hwnd,hAudioResource,a);
      if(hwndMenu) {
        if(WinPopupMenu(hwnd,hwnd,hwndMenu,ptl.x,ptl.y,b,PU_VCONSTRAIN|PU_HCONSTRAIN|
                        PU_MOUSEBUTTON1|PU_KEYBOARD|PU_NONE|PU_SELECTITEM)) {
										
          return (MRESULT)FALSE;
        }
      }
      break;
    case WM_TIMER:
      if(SHORT1FROMMP(mp1)==1) {
        thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
        /* Calculate the play time of the waves and mp3 in another thread */
        _beginthread(playTimeThreadFunc,NULL,8192*16,(void*)thisPtr); //Fehlerbehandlung fehlt    
        WinStopTimer(	WinQueryAnchorBlock(HWND_DESKTOP),hwnd,1);
      }
      return (MRESULT) TRUE;			
    case WM_COMMAND:	
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        switch(SHORT1FROMMP(mp1))
          {
          case IDM_STATUSDEFAULTHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(603,AFHELPLIBRARY);
          default:
            break;
          }
      }/* end of if(thisPtr) */
			
      return (MRESULT) TRUE;
    default:
      if(msg == atomUpdateStatusbar){ /* Update the statusbar */
        WinStartTimer(WinQueryAnchorBlock(HWND_DESKTOP),hwnd,1,500);
        return (MRESULT) TRUE;
      }
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}


/****************************************************************************/
/*                                                                          */
/* This procedure handles the select-dialog (frame extension) of the folder */
/*                                                                          */
/****************************************************************************/
MRESULT EXPENTRY selectDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  CWAudioFolder *thisPtr;
  HWND hwndTracks;
  WPObject * contentObject;
  char name[CCHMAXPATH];
  ULONG ulNameSize;
  HWND hwndCnr;
  PMINIRECORDCORE mrc;
  SIZEL sizel;
  ULONG ulFlags;
  HWND hwndTemp;
  POINTL ptl;
  SHORT a,b;
    
  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(hwnd,QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      thisPtr=(CWAudioFolder*) PVOIDFROMMP(mp2);
      WinShowWindow(WinWindowFromID(hwnd,IDEF_TRACKNAME),thisPtr->bShowGrab);
      WinShowWindow(WinWindowFromID(hwnd,IDST_TRACKNAME),thisPtr->bShowGrab);
      WinCheckButton(hwnd,IDRB_GRAB,thisPtr->bShowGrab);
      WinCheckButton(hwnd,IDRB_WRITE,!thisPtr->bShowGrab);
      if(thisPtr->bShowGrab) {
        if(thisPtr->tid) {
          /* There is a grabbing in progress */
          thisPtr->cwEnableSelectControls(FALSE);
        }
      }
      return (MRESULT) TRUE;
    case WM_HELP:
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr)
        return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_GRABDIALOG,AFHELPLIBRARY);
      break;
    case WM_DESTROY:
      /* The dialog closes and gets destroyed */     
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        //thisPtr->wpSaveDeferred();
      }
      break;    
    case WM_BUTTON2DOWN:
      WinQueryPointerPos(HWND_DESKTOP,&ptl);
      WinMapWindowPoints(HWND_DESKTOP,hwnd,&ptl,1);
      hwndTracks=WinWindowFromPoint(hwnd,&ptl,FALSE);/* Get child window under ptr */
      switch(WinQueryWindowUShort(hwndTracks,QWS_ID))/* Get child window id */
        {

        default:
          a=IDM_DEFAULTHELP;
          b=a;
          break;
        }
      hwndTracks=WinLoadMenu(hwnd,hAudioResource,a);
      if(hwndTracks) {
        if(WinPopupMenu(hwnd,hwnd,hwndTracks,ptl.x,ptl.y,b,PU_VCONSTRAIN|PU_HCONSTRAIN|
                        PU_MOUSEBUTTON1|PU_KEYBOARD|PU_NONE|PU_SELECTITEM)) {
                                        
          return (MRESULT)FALSE;
        }
      }
      break;

    case WM_CONTROL:
      /* This part handles the radio button messages */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        switch(SHORT1FROMMP(mp1))
          {
          case IDRB_WRITE:
            /* Hide trackname window */
            WinShowWindow(WinWindowFromID(hwnd,IDEF_TRACKNAME),FALSE);
            WinShowWindow(WinWindowFromID(hwnd,IDST_TRACKNAME),FALSE);
            hwndTemp=thisPtr->cwQueryFrameCtl(WinQueryWindow(hwnd,QW_PARENT), &sizel,FCTL_LEFT, &ulFlags);
                        
            if(WinQueryWindowUShort(hwndTemp,QWS_ID)==IDDLG_GRABDIALOG)
              thisPtr->hwndGrab=hwndTemp;
            if(!thisPtr->hwndWrite)
              thisPtr->hwndWrite=WinLoadDlg(WinQueryWindow(hwnd,QW_PARENT),WinQueryWindow(hwnd,QW_PARENT),
                                            dialogProc,hAudioResource,ID_DIALOG,thisPtr);
            thisPtr->cwAddFrameCtl(WinQueryWindow(hwnd,QW_PARENT),thisPtr->hwndWrite,sizel,FCTL_LEFT, ulFlags);
            WinShowWindow(thisPtr->hwndGrab,FALSE);
                        
            thisPtr->cwUpdateFrame(WinQueryWindow(hwnd,QW_PARENT));
            break;
          case IDRB_GRAB:
            WinShowWindow(WinWindowFromID(hwnd,IDEF_TRACKNAME),TRUE);
            WinShowWindow(WinWindowFromID(hwnd,IDST_TRACKNAME),TRUE);
            hwndTemp=thisPtr->cwQueryFrameCtl(WinQueryWindow(hwnd,QW_PARENT), &sizel,FCTL_LEFT, &ulFlags);
            if(WinQueryWindowUShort(hwndTemp,QWS_ID)==ID_DIALOG)
              thisPtr->hwndWrite=hwndTemp;
            if(!thisPtr->hwndGrab) {
              thisPtr->hwndGrab=WinLoadDlg(WinQueryWindow(hwnd,QW_PARENT),WinQueryWindow(hwnd,QW_PARENT),
                                           grabDialogProc,hAudioResource,IDDLG_GRABDIALOG,thisPtr);
            }
            thisPtr->cwAddFrameCtl(WinQueryWindow(hwnd,QW_PARENT),thisPtr->hwndGrab, sizel,FCTL_LEFT, ulFlags);
            WinShowWindow(thisPtr->hwndWrite,FALSE);
            thisPtr->cwUpdateFrame(WinQueryWindow(hwnd,QW_PARENT)); 
            break;
          default:
            break;
          }
        /* Tell the WPS to save the instance data */
        //thisPtr->wpSaveDeferred();
      }/* if(thisPtr) */
      break;
    case WM_COMMAND:    
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        switch(SHORT1FROMMP(mp1))
          {
          case IDM_DEFAULTHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(600,AFHELPLIBRARY);
          default:
            break;
          }
      }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}

void daoDisableCB(HWND hwnd, BOOL disable) 
{
  WinEnableWindow(WinWindowFromID(hwnd,IDCB_PAD),!disable);
  WinEnableWindow(WinWindowFromID(hwnd,IDCB_NOFIX),!disable);
}

/**************************************************************************/
/*                                                                        */
/* This procedure handles the main dialog (frame extension) of the folder */
/*                                                                        */
/**************************************************************************/
MRESULT EXPENTRY dialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  CWAudioFolder *thisPtr;
  HWND hwndTracks;
  WPObject * contentObject;
  char name[CCHMAXPATH];
  char text[CCHMAXPATH];
  ULONG ulNameSize;
  HWND hwndCnr;
  PMINIRECORDCORE mrc;
  ULONG ulWriteFlagsLocal;
  ULONG ulSize;
  POINTL ptl;
  SHORT a,b;
    
  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(hwnd,QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      thisPtr=(CWAudioFolder*) PVOIDFROMMP(mp2);
      /* Set the check buttons according the instance write flags */
      /* The member function cwQueryWriteFlags() returns the current
         state of the flags. */
      ulWriteFlagsLocal=thisPtr->cwQueryWriteFlags();
      WinCheckButton(hwnd,IDCB_PAD,(ulWriteFlagsLocal & IDWF_PAD)&&TRUE);
      WinCheckButton(hwnd,IDCB_NOFIX,(ulWriteFlagsLocal & IDWF_NOFIX)&&TRUE);
      WinCheckButton(hwnd,IDCB_DUMMY,(ulWriteFlagsLocal & IDWF_DUMMY)&&TRUE);
      WinCheckButton(hwnd,IDCB_PREEMP,(ulWriteFlagsLocal & IDWF_PREEMP)&&TRUE);
      WinCheckButton(hwnd,IDCB_DAO,(ulWriteFlagsLocal & IDWF_DAO)&&TRUE);
      if(ulWriteFlagsLocal & IDWF_DAO) {
        daoDisableCB(hwnd, TRUE);
      }
      return (MRESULT) TRUE;
    case WM_HELP:
      /* Get the object pointer from the window words. We saved it in WM_INITDLG. */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr)
        /* Display the helppanel IDDLG_GRABDIALOG */
        return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_GRABDIALOG,AFHELPLIBRARY);
      break;
    case WM_DESTROY:
      /* The dialog closes and gets destroyed */
      /* Get the object pointer */     
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        /* Query the checkboxes */
        if(WinQueryButtonCheckstate(hwnd,IDCB_PAD))
          ulWriteFlagsLocal=IDWF_PAD;
        if(WinQueryButtonCheckstate(hwnd,IDCB_NOFIX))
          ulWriteFlagsLocal|=IDWF_NOFIX;
        if(WinQueryButtonCheckstate(hwnd,IDCB_DUMMY))
          ulWriteFlagsLocal|=IDWF_DUMMY;
        if(WinQueryButtonCheckstate(hwnd,IDCB_PREEMP))
          ulWriteFlagsLocal|=IDWF_PREEMP;
        if(WinQueryButtonCheckstate(hwnd,IDCB_DAO))
          ulWriteFlagsLocal|=IDWF_DAO;
        /* Save checkbox states in the instance varables */
        thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,WF_ALLFLAGS);
        /* Let the WPS save the instance data */
        thisPtr->wpSaveDeferred();
        thisPtr->hwndWrite=0;
      }
      break;    
    case WM_BUTTON2DOWN:
      WinQueryPointerPos(HWND_DESKTOP,&ptl);
      WinMapWindowPoints(HWND_DESKTOP,hwnd,&ptl,1);
      hwndTracks=WinWindowFromPoint(hwnd,&ptl,FALSE);/* Get child window under ptr */
      switch(WinQueryWindowUShort(hwndTracks,QWS_ID))/* Get child window id */
        {
        case IDCB_PAD:
          a=IDM_WRITEPADHELP;
          b=a;
          break;
        case IDCB_NOFIX:
          a=IDM_WRITENOFIXHELP;
          b=a;
          break;
        case IDCB_DUMMY:
          a=IDM_WRITEDUMMYHELP;
          b=a;
          break;
        case IDCB_PREEMP:
          a=IDM_WRITEPREEMPHELP;
          b=a;
          break;
        case IDCB_DAO:
          a=IDM_WRITEDAOHELP;
          b=a;
          break;
        case IDPB_BURN:
          a=IDM_WRITEBURNHELP;
          b=a;
          break;

        default:
          a=IDM_WRITEDEFAULTHELP;
          b=IDM_CDRECORDSETUP;
          break;
        }
      hwndTracks=WinLoadMenu(hwnd,hAudioResource,a);
      if(hwndTracks) {
        if(WinPopupMenu(hwnd,hwnd,hwndTracks,ptl.x,ptl.y,b,PU_VCONSTRAIN|PU_HCONSTRAIN|
                        PU_MOUSEBUTTON1|PU_KEYBOARD|PU_NONE|PU_SELECTITEM)) {
                                        
          return (MRESULT)FALSE;
        }
      }
      break;
    case WM_CONTROL:
      /* This part handles the check button messages */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        ulWriteFlagsLocal=0;
        switch(SHORT1FROMMP(mp1))
          {
          case IDCB_PAD:
            if(WinQueryButtonCheckstate(hwnd,IDCB_PAD))
              ulWriteFlagsLocal=IDWF_PAD;
            thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,IDWF_PAD);
            break;
          case IDCB_NOFIX:
            if(WinQueryButtonCheckstate(hwnd,IDCB_NOFIX))
              ulWriteFlagsLocal=IDWF_NOFIX;
            thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,IDWF_NOFIX);
            break;
          case IDCB_DUMMY:
            if(WinQueryButtonCheckstate(hwnd,IDCB_DUMMY))
              ulWriteFlagsLocal=IDWF_DUMMY;
            thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,IDWF_DUMMY);
            break;
          case IDCB_PREEMP:
            if(WinQueryButtonCheckstate(hwnd,IDCB_PREEMP))
              ulWriteFlagsLocal=IDWF_PREEMP;
            thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,IDWF_PREEMP);
            break;
          case IDCB_DAO:
            if(WinQueryButtonCheckstate(hwnd,IDCB_DAO)) {
              ulWriteFlagsLocal=IDWF_DAO;
              daoDisableCB(hwnd, TRUE);
            }
            else
              daoDisableCB(hwnd, FALSE);
            thisPtr->cwSetWriteFlags( ulWriteFlagsLocal,IDWF_DAO);
            thisPtr->cwForceStatusUpdate();
            break;
          default:
            break;
          }
        /* Tell the WPS to save the instance data */
        thisPtr->wpSaveDeferred();
      }
      break;
    case WM_COMMAND:    
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr) {
        switch(SHORT1FROMMP(mp1))
          {
          case IDM_WRITEDEFAULTHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(601,AFHELPLIBRARY);
          case IDM_WRITEPADHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITEPAD,AFHELPLIBRARY);
          case IDM_WRITENOFIXHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITENOFIX,AFHELPLIBRARY);
          case IDM_WRITEDUMMYHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITEDUMMY,AFHELPLIBRARY);
          case IDM_WRITEPREEMPHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITEPREEMP,AFHELPLIBRARY);
          case IDM_WRITEDAOHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITEDAO,AFHELPLIBRARY);
          case IDM_WRITEBURNHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_WRITEBURN,AFHELPLIBRARY);            
          case IDM_CDRECORDSETUP:
            bGrabberFirst=FALSE;
            thisPtr->wpViewObject(NULL,OPEN_SETTINGS,0);
            return (MRESULT)TRUE;
          default:
            break;
          }
      }
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_BURN:
          ulSize=0;
          /* Write CD button pressed */
          if(pvAudioSharedMem) {
            WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP,"There is already a CD writing process running." ,"Write audio tracks",
                            0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
            break;
          }
          thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
          if(!setupDone){
            if(thisPtr)thisPtr->wpViewObject(NULL,OPEN_SETTINGS,0);
          }
          else{
            /* Load the track list dialog */
            hwndTracks=WinLoadDlg(HWND_DESKTOP,hwnd,trackDialogProc,hAudioResource,IDDLG_SHOWTITLES,thisPtr);
            if(hwndTracks){ /* Catch error */
              /* Get the object pointer */
              thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
              if(thisPtr){ /* Catch error */
                /* Get folder hwnd */
                hwndCnr=WinQueryWindow(hwnd,QW_PARENT);
                thisPtr->cwFillTrackList(hwndCnr,WinWindowFromID(hwndTracks,IDLB_TRACKLIST));
                ulSize=thisPtr->ulTrackSize;
                /* Text: "Total size: %d.%d.%d Bytes" */
                getMessage(text,IDSTR_TOTALSIZE,sizeof(text), hAudioResource,hwnd);
                sprintf(name,text,ulSize/1000000,(ulSize%1000000)/1000,(ulSize%1000));
                WinSetWindowText(WinWindowFromID(hwndTracks,IDST_TOTALSIZE),name);
            
                ulSize/=(2*2*44100);/* Calculate seconds */
                /* Text: "Total time: %d min %d sec" */
                getMessage(text,IDSTR_TOTALTIME,sizeof(text), hAudioResource,hwnd);
                sprintf(name,text, (ulSize-(ulSize % 60))/60 , (ulSize % 60));
                WinSetWindowText(WinWindowFromID(hwndTracks,IDST_TOTALTIME),name);
              }//end of if(thisPtr)
            }//end of if(hwndTracks)
          }
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}

/**************************************************************************/
/*                                                                        */
/* This procedure handles the grab-dialog (frame extension) of the folder */
/*                                                                        */
/**************************************************************************/
MRESULT EXPENTRY grabDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  CWAudioFolder *thisPtr;
  int numTracks;
  HFILE hfCD;
  char text[CCHMAXPATH*2];
  char title[CCHMAXPATH];
  ULONG ulSize;    
  SHORT a,b;
  ULONG ulFlags;
  SIZEL sizel;
  HWND hwndTemp;
  POINTL ptl;
    
  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(hwnd,QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      thisPtr=(CWAudioFolder*) PVOIDFROMMP(mp2);
      WinEnableWindow(WinWindowFromID(hwnd,IDPB_SELECTALL) ,FALSE);
      WinEnableWindow(WinWindowFromID(hwnd,IDPB_GRAB) ,FALSE);
      return (MRESULT) TRUE;
    case WM_DESTROY:
      /* The dialog closes and gets destroyed */     
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        thisPtr->hwndGrab=0;
      }
      break;    
    case WM_HELP:
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr)
        return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_GRABDIALOG,AFHELPLIBRARY);
      break;
    case WM_CONTROL:
      /* This part handles the radio button messages */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr){
        switch(SHORT1FROMMP(mp1))
          {
          default:                     
            break;
          }
        /* Tell the WPS to save the instance data */
      }
      break;
    case WM_BUTTON2DOWN:
      WinQueryPointerPos(HWND_DESKTOP,&ptl);
      WinMapWindowPoints(HWND_DESKTOP,hwnd,&ptl,1);
      hwndTemp=WinWindowFromPoint(hwnd,&ptl,FALSE);/* Get child window under ptr */
      switch(WinQueryWindowUShort(hwndTemp,QWS_ID))/* Get child window id */
        {
        case IDLB_GRABTRACKS:
          a=IDM_GRABTRACKLISTHELP;
          b=IDPB_REFRESH;
          break;
        case IDPB_GRAB:
          a=IDM_GRABGRABHELP;
          b=a;
          break;
        case IDPB_REFRESH:
          a=IDM_GRABREFRESHHELP;
          b=a;
          break;
        case IDPB_SELECTALL:
          a=IDM_GRABSELECTALLHELP;
          b=a;
          break;
        default:
          a=IDM_GRABDEFAULTHELP;
          b=IDM_GRABBERSETUP;
          break;
        }
      hwndTemp=WinLoadMenu(hwnd,hAudioResource,a);
      if(hwndTemp) {
        if(WinPopupMenu(hwnd,hwnd,hwndTemp,ptl.x,ptl.y,b,PU_VCONSTRAIN|PU_HCONSTRAIN|
                        PU_MOUSEBUTTON1|PU_KEYBOARD|PU_NONE|PU_SELECTITEM)) {
          if(!(a==IDM_GRABTRACKLISTHELP && WinIsControlEnabled(hwnd,IDPB_SELECTALL)))
            WinEnableMenuItem(hwndTemp,IDPB_SELECTALL,FALSE);
                    
          return (MRESULT)FALSE;
        }
      }
      break;
    case WM_COMMAND:    
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(thisPtr) {
        switch(SHORT1FROMMP(mp1))
          {
            /* These are the popup menu items */
          case IDM_GRABDEFAULTHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(602,AFHELPLIBRARY);
            break;
          case IDM_GRABGRABHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_GRABGRAB,AFHELPLIBRARY);
            break;
          case IDM_GRABTRACKLISTHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_GRABTRACKLIST,AFHELPLIBRARY);
            break;                  
          case IDM_GRABREFRESHHELP:                   
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_GRABREFRESH,AFHELPLIBRARY);
            break;
          case IDM_GRABSELECTALLHELP:
            return (MRESULT)thisPtr->wpDisplayHelp(IDHLP_GRABSELECTALL,AFHELPLIBRARY);
            break;
          case IDM_GRABBERSETUP:
            bGrabberFirst=TRUE;
            thisPtr->wpViewObject(NULL,OPEN_SETTINGS,0);
            return (MRESULT)TRUE;
            /* push buttons */
          default:
            break;
          }
      } /* end of if(thisPtr) */
                        
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_GRAB:
          thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
          if(thisPtr) {            
            hwndTemp=thisPtr->cwQueryFrameCtl(WinQueryWindow(hwnd,QW_PARENT), &sizel,FCTL_TOP, &ulFlags);
                    
            WinQueryWindowText( WinWindowFromID(hwndTemp,IDEF_TRACKNAME),sizeof(title),title);
                        
            if(!strlen(title)) {
              /* Text:  "You must specify a filename."
                 Title: "Grab tracks message"
                 */
              messageBox( text, IDSTR_NOTRACKNAMETEXT , sizeof(text),
                          title, IDSTR_NOTRACKSSELECTEDTITLE, sizeof(title),
                          hAudioResource, hwnd, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
              break;
            }
            /* Get num Tracks */
            numTracks=(int)LONGFROMMR(WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_QUERYITEMCOUNT,0,0));
            if(!numTracks)
              break;/* Catch error */                        
            a=SHORT1FROMMR(WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_QUERYSELECTION,MPFROMSHORT(LIT_FIRST),0));
            if(a==LIT_NONE) {
              /*
                Text: "There are no tracks selected"
                Title: "Grab tracks message"                           
                */             
              messageBox( text, IDSTR_NOTRACKSSELECTEDTEXT , sizeof(text),
                          title, IDSTR_NOTRACKSSELECTEDTITLE, sizeof(title),
                          hAudioResource, hwnd, MB_OK | MB_INFORMATION | MB_MOVEABLE);
              break;
            }
            
            sprintf(text,"\"%s\"",title);
            while(a!=LIT_NONE) {
              sprintf(title," %d",a+1);
              strcat(text,title);
              a=SHORT1FROMMR(WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_QUERYSELECTION,MPFROMSHORT(a),0));
            };
         
            /* Query folder name */
            ulSize=sizeof(title);
            thisPtr->wpQueryRealName(title,&ulSize,TRUE);
            
            launchPMWrapper(text,title, "pacdgrab.exe","Grab Tracks PM");
            break;
          }/* end of if(thisPtr) */
          break;
        case IDPB_SELECTALL:
          numTracks=(int)LONGFROMMR(WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_QUERYITEMCOUNT,0,0));
          if(!numTracks)
            break;
          for(a=0;a<numTracks;a++) {
            WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_SELECTITEM,MPFROMSHORT(a),MPFROMSHORT(TRUE));
          }
          break;
        case IDPB_REFRESH:
          int numTracks;
          int a;
          LONG lCddbId;
          CDDBINFO cddbInfo;

          thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
          if(thisPtr) {
            if(!GrabberSetupDone){
              thisPtr->wpViewObject(NULL,OPEN_SETTINGS,0);
              break;
            }
                    
            WinSendMsg(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LM_DELETEALL,0,
                       0);
            WinEnableWindow(WinWindowFromID(hwnd,IDPB_SELECTALL) ,FALSE);
            WinEnableWindow(WinWindowFromID(hwnd,IDPB_GRAB) ,FALSE);
            hfCD=openDrive(chosenCD);
            if(hfCD) {
              numTracks=(int)CDQueryAudioCDTracks(hfCD);
              if(numTracks) {
                if(numTracks==-1) {
                  /*
                    Text: "The Disk is a data disk!"
                    Title: "Track grabbing"                       
                    */             
                  messageBox( text, IDSTR_ERRORISDATACDTEXT , sizeof(text),
                              title, IDSTR_ERRORISDATACDTITLE, sizeof(title),
                              hAudioResource, hwnd, MB_OK | MB_ICONEXCLAMATION | MB_MOVEABLE);
                }
                else {
                  /* Insert the items into the listbox */
                  lCddbId=CDDBDiscID(chosenCD,&cddbInfo);
                  /* Message: "CDDB-ID: %X, Querying local database..." */
                  getMessage(title,IDSTR_CDDBQUERYLOCALDB, sizeof(title), hAudioResource,hwnd);
                  sprintf(text,title,lCddbId);

                  /* Open CDDB.ini and read track names */
                  
                  for(a=1;a<=numTracks;a++) {
                    getTrackName(lCddbId,a,text);             
                    WinInsertLboxItem(WinWindowFromID(hwnd,IDLB_GRABTRACKS),LIT_END,
                                      text);
                  }/* end of for */
                  WinEnableWindow(WinWindowFromID(hwnd,IDPB_SELECTALL) ,TRUE);
                  WinEnableWindow(WinWindowFromID(hwnd,IDPB_GRAB) , TRUE);
                }
              }// end of if(!numTracks)                       
              closeDrive(hfCD);
            }
          }/* (if thisPtr) */
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}

/************************************************************************************/


/***********************************************/
/*                                             */
/* This procedure handles the tracklist dialog */
/*                                             */
/***********************************************/
MRESULT EXPENTRY trackDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  CWAudioFolder *thisPtr;
  char * text;
  ULONG ulBufferSize;
  ULONG ulFlags;
  char name[CCHMAXPATH];
  char anzahl[100];
  int b;
  LONG a;
  ULONG rc;

  switch(msg)
    {
    case WM_INITDLG :	
      WinSetWindowULong(hwnd,QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      /* Check if we have items in the listbox. If not disable Ok-button. */
      thisPtr=(CWAudioFolder*)PVOIDFROMMP(mp2);
      thisPtr->hwndWriteLB=WinWindowFromID(hwnd,IDLB_TRACKLIST);
      /* Disable windows in write dialog */
      thisPtr->cwEnableWriteControls(FALSE);
      /* Show Warning text if in DAO mode */ 
      ulFlags=thisPtr->cwQueryWriteFlags();
      if(ulFlags&IDWF_DAO)
        WinShowWindow(WinWindowFromID(hwnd,IDST_MP3DISCARDED),TRUE);
      else
        WinShowWindow(WinWindowFromID(hwnd,IDST_MP3DISCARDED),FALSE);
      return (MRESULT) TRUE;
    case WM_DESTROY:
      /* The dialog closes and gets destroyed */     
      /* Enable windows in write dialog */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      thisPtr->hwndWriteLB=0;
      thisPtr->cwEnableWriteControls(TRUE);
      break;	
    case WM_CLOSE:
      /* Enable the 'Write'-button of the Audio folder */
      WinDismissDlg(hwnd,0);
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
      if(!thisPtr->tidWrite)
        WinDestroyWindow(hwnd);
      break;	
    case WM_COMMAND:	
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_CANCEL:
          thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
          thisPtr->cwEnableWriteControls(TRUE);
          WinPostMsg(hwnd,WM_CLOSE,0,0);
          break;
        case IDPB_OK:
          /*******************************************************************/
          if(!pvAudioSharedMem) {
            if(DosAllocSharedMem(&pvAudioSharedMem,"\\SHAREMEM\\AUDIOCMDLINE",AUDIOSHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
              WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, "Can't allocate shared Mem!","Write audio tracks",
                              0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
              WinPostMsg(hwnd,WM_CLOSE,0,0);
              break;
            }
          }
          memset(pvAudioSharedMem,0,AUDIOSHAREDMEM_SIZE);
          text= (char*)pvAudioSharedMem; 
          thisPtr=(CWAudioFolder*) WinQueryWindowULong(hwnd,QWL_USER);
          if(!thisPtr) {
            DosFreeMem(pvAudioSharedMem);
            pvAudioSharedMem=NULL;
            WinPostMsg(hwnd,WM_CLOSE,0,0);
            break;
          }
          ulFlags=thisPtr->cwQueryWriteFlags();

          /* Query the full path of our folder*/
          ulBufferSize=sizeof(name);
          thisPtr->wpQueryRealName(name,&ulBufferSize,TRUE);

          /* Put cdrecord path in the shared mem if DAO mode */
          /* TAO (writeaud.exe) currently does not need it */ 

          /* Build DAO data */
          if(ulFlags&IDWF_DAO) {
            sprintf(text,"%s| ",chrCdrdaoPath);/* Add '|' to mark end of path for helper */
            strcat(text,buildAudioWriteParamDAO(thisPtr));/* Add audio params for DAO */
            strcat(text,name);/* Name of this audio folder */
            strcat(text,"\\default.toc");/* add toc file name*/
            buildTocFile( thisPtr, thisPtr->hwndWriteLB);
          }
          else{ /* TAO with CDRecord/2 */
            strcat(text, buildAudioWriteParam(thisPtr,""));                       
            /* Add the tracknames */
            a=SHORT1FROMMR(WinSendMsg(thisPtr->hwndWriteLB,LM_QUERYITEMCOUNT, NULL, NULL));
            for(b=0;b<a;b++) {
              if(SHORT1FROMMR(WinSendMsg(thisPtr->hwndWriteLB,LM_QUERYITEMTEXT,MPFROM2SHORT((SHORT)b,(SHORT)CCHMAXPATH),&name))) {
                strcat(text," \"");
                strcat(text,name);
                strcat(text,"\"");
              }
            }
          } /* else{  TAO with CDRecord/2 */

          /* Query the full path of our folder*/
          ulBufferSize=sizeof(name);
          thisPtr->wpQueryRealName(name,&ulBufferSize,TRUE);

          /* Add number of audio files and total size of all tracks to parameters so the helper
             prog has not to query it */          
          sprintf(anzahl,"%d %d",a,thisPtr->ulTrackSize);          

          if(ulFlags&IDWF_DAO) 
            rc=launchPMWrapper(anzahl, name,"pmauddao.exe","Writing audio tracks (DAO)");
          else
            rc=launchPMWrapper(anzahl,name,"pmwrtaud.exe","Writing audio tracks (TAO)");
          if(rc) {
            DosFreeMem(pvAudioSharedMem);
            pvAudioSharedMem=NULL;
          }
          WinPostMsg(hwnd,WM_CLOSE,0,0);
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      return WinDefDlgProc(hwnd, msg, mp1, mp2);
    }
  return (MRESULT) FALSE;
}





/***********************************************************************/

//        procedures for settings pages

/***********************************************************************/


/***********************************************************************/
/*                                                                     */
/* This procedure handles the cdrecord settings page for audio folders */
/* The only difference to the data folder is the option entryfield.    */
/* It is possible to specify different options for data and audio      */
/* since version 0.34                                                  */
/*                                                                     */
/***********************************************************************/
MRESULT EXPENTRY audioCdrecordOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  FILEDLG fd = { 0 };
  CWAudioFolder* thisPtr;
  ULONG ulFlags;
  int a;
  char chrCD[4];
  char driveID[40];
  char text[CCHMAXPATH];

  HOBJECT  hObject;

  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(WinWindowFromID(hwnd,IDPB_CDRECORDBROWSE),
                        QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      WinSendMsg(WinWindowFromID(hwnd,IDEF_CDRECORDPATH),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)CCHMAXPATH),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRECORDPATH),chrCDRecord);
      WinSendMsg(WinWindowFromID(hwnd,IDEF_CDRECORDOPTIONS),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)CCHMAXPATH),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRECORDOPTIONS),chrAudioCDROptions);
      for( a=0; a<iNumCD ;a++) {
        chrCD[0]=cFirstCD+a;
        chrCD[1]=':';
        chrCD[2]=0;
        WinSendMsg(WinWindowFromID(hwnd,IDLB_WRITERLETTER),LM_INSERTITEM,MPFROMSHORT(LIT_END),(MPARAM)chrCD);
        if(!a){
          if(!setupDone)
            WinSetWindowText(WinWindowFromID(hwnd,IDLB_WRITERLETTER),chrCD);
          else
            WinSetWindowText(WinWindowFromID(hwnd,IDLB_WRITERLETTER),chosenWriter);
        }
      }
      /* Change the name of the options text field */
      getMessage(text,IDSTRA_CDROPTIONS,sizeof(text), hAudioResource,hwnd);
      WinSetWindowText(WinWindowFromID(hwnd,IDST_CDRECORDOPTIONS),text);
      return (MRESULT) TRUE;
    case WM_HELP:
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_CDRECORDBROWSE),QWL_USER);
      if(!thisPtr)
        break;
      switch(WinQueryWindowUShort(WinQueryFocus(HWND_DESKTOP),QWS_ID))
        {
        case IDEF_CDRECORDOPTIONS:
          return (MRESULT)thisPtr->wpDisplayHelp(IDEF_CDRECORDOPTIONS,AFHELPLIBRARY);
        case IDPB_WIDERRUFEN:
          return (MRESULT)thisPtr->wpDisplayHelp(IDPB_WIDERRUFEN,AFHELPLIBRARY);
        case IDPB_CDRECORDBROWSE:
          return (MRESULT)thisPtr->wpDisplayHelp(IDPB_CDRECORDBROWSE,AFHELPLIBRARY);
        case IDEF_CDRECORDPATH:
          return (MRESULT)thisPtr->wpDisplayHelp(IDEF_CDRECORDPATH,AFHELPLIBRARY);
        default:
          break;
        }
      return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_CDRECORDSETUP,AFHELPLIBRARY);
      break;
    case WM_DESTROY:
      /* The notebook closes and gets destroyed */
      /* Query the cdrecord path from the entryfield */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_CDRECORDPATH),sizeof(chrCDRecord),chrCDRecord);
      /* Query the cdrecord options from the entryfield for audio folders */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_CDRECORDOPTIONS),sizeof(chrAudioCDROptions),chrAudioCDROptions);
      sendConfig();

      /* Set/unset writeflag of diskobject */
      /*      for( a=0; a<iNumCD ;a++) {
              chrCD[0]=cFirstCD+a;
              chrCD[1]=0;*/
      /* Build ID of CD-Drive object */
      //sprintf(driveID,"<WP_DRIVE_%s>",chrCD);
      /* Clear writer flag of disk objects */
      //if((hObject=WinQueryObject(driveID))!=NULL) {
      //  WinSetObjectData(hObject,"ISWRITER=0");
      // }
      //}               
      /* Query the cd-drive */
      WinQueryWindowText( WinWindowFromID(hwnd,IDLB_WRITERLETTER),sizeof(chosenWriter),chosenWriter);
      //    WinQueryWindowText( WinWindowFromID(hwnd,IDLB_WRITERLETTER),sizeof(chrCD),chrCD);
      // chrCD[1]=0;
      /* Build ID of chosenCD-Drive object */
      // sprintf(driveID,"<WP_DRIVE_%s>",chrCD);
      /* Set writer flag of this disk object */
      // if((hObject=WinQueryObject(driveID))!=NULL) {
      //        WinSetObjectData(hObject,"ISWRITER=1");
      //}
      /* Let the WPS save the new data */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_CDRECORDBROWSE),QWL_USER);
      if(thisPtr)
        thisPtr->wpSaveImmediate();      
      /* Setup is done */
      setupDone=TRUE;
      return (MRESULT) TRUE;
    case WM_COMMAND:    
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_CDRECORDBROWSE:
          /* User pressed the browse button */
          fd.cbSize = sizeof( fd );
          /* It's an centered 'Open'-dialog */
          fd.fl = FDS_OPEN_DIALOG|FDS_CENTER;
          /* Title: "Search CDRecord/2" */
          getMessage(text,IDSTR_FDLGSEARCHCDR2TITLE,sizeof(text), hAudioResource,hwnd);
          /* Set the title of the file dialog */
          fd.pszTitle = text;
          /* Only show *.exe files */
          sprintf(fd.szFullFile,"%s","*.exe");

          if( WinFileDlg( HWND_DESKTOP, hwnd, &fd ) == NULLHANDLE )
            {
              /* WinFileDlg failed */
              break;
            }
          if( fd.lReturn == DID_OK )
            {
              WinSetWindowText( WinWindowFromID(hwnd,IDEF_CDRECORDPATH), fd.szFullFile );
            }
          break;
        case IDPB_WIDERRUFEN:
          /* User pressed the UNDO button */
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRECORDPATH),chrCDRecord);
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRECORDOPTIONS),chrAudioCDROptions);
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}

/***********************************************************************/
/*                                                                     */
/* This procedure handles the general settings page for creator folders*/
/*                                                                     */
/***********************************************************************/
MRESULT EXPENTRY generalOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  FILEDLG fd = { 0 };
  CWAudioFolder* thisPtr;
  ULONG ulFlags;
  int a;
  char chrCD[4];
  char driveID[40];
  char text[CCHMAXPATH];

  HOBJECT  hObject;

  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(WinWindowFromID(hwnd,IDPB_SCAN),
                        QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      WinSetWindowText(WinWindowFromID(hwnd,IDDD_DEVICESELECTION),chrDeviceName);
      /* Device Spinbuttons */
      /*      WinSendMsg(WinWindowFromID(hwnd,IDSB_BUS),SPBM_SETLIMITS,MPFROMLONG(15),MPFROMLONG(0));
              WinSendMsg(WinWindowFromID(hwnd,IDSB_TARGET),SPBM_SETLIMITS,MPFROMLONG(15),MPFROMLONG(0));
              WinSendMsg(WinWindowFromID(hwnd,IDSB_LUN),SPBM_SETLIMITS,MPFROMLONG(15),MPFROMLONG(0));
              WinSendMsg(WinWindowFromID(hwnd,IDSB_BUS),SPBM_SETCURRENTVALUE,(MPARAM) iBus,MPFROMLONG(0));
              WinSendMsg(WinWindowFromID(hwnd,IDSB_TARGET),SPBM_SETCURRENTVALUE,(MPARAM)iTarget,MPFROMLONG(0));
              WinSendMsg(WinWindowFromID(hwnd,IDSB_LUN),SPBM_SETCURRENTVALUE,(MPARAM)iLun,MPFROMLONG(0));*/
      /* Speed spin button */
      WinSendMsg(WinWindowFromID(hwnd,IDSB_SPEED),SPBM_SETLIMITS,MPFROMLONG(16),MPFROMLONG(1));
      WinSendMsg(WinWindowFromID(hwnd,IDSB_SPEED),SPBM_SETCURRENTVALUE,(MPARAM)iSpeed,MPFROMLONG(0));
      /* FIFO spin button */
      WinSendMsg(WinWindowFromID(hwnd,IDSB_FIFO),SPBM_SETLIMITS,MPFROMLONG(64),MPFROMLONG(4));
      WinSendMsg(WinWindowFromID(hwnd,IDSB_FIFO),SPBM_SETCURRENTVALUE,(MPARAM)iFifo,MPFROMLONG(0));

      if(lCDROptions&IDCDR_HIDEWINDOW)
        WinCheckButton(hwnd,IDCB_CDRSTARTHIDDEN,1);
      if(lCDROptions&IDCDR_CLOSEWINDOW)
        WinCheckButton(hwnd,IDCB_CDRCLOSEWINDOW,1);

      return (MRESULT) TRUE;
    case WM_HELP:
      break;
    case WM_APPTERMINATENOTIFY:
      switch(LONGFROMMP(mp1)) {
      case ACKEY_SCANBUS:
        break;
      default:
        break;
      }/* switch */
      break; 
    case WM_DESTROY:
      /* The notebook closes and gets destroyed */
      WinQueryWindowText( WinWindowFromID(hwnd,IDDD_DEVICESELECTION),sizeof(chrDeviceName),chrDeviceName);
      sscanf(chrDeviceName,"%d,%d,%d ",&iBus,&iTarget,&iLun);
      /*WinSendMsg(WinWindowFromID(hwnd,IDSB_BUS),SPBM_QUERYVALUE,MPFROMP(&ulFlags),
        MPFROM2SHORT(0,SPBQ_ALWAYSUPDATE));
        iBus=ulFlags;
        WinSendMsg(WinWindowFromID(hwnd,IDSB_TARGET),SPBM_QUERYVALUE,MPFROMP(&ulFlags),
        MPFROM2SHORT(0,SPBQ_ALWAYSUPDATE));
        iTarget=ulFlags;
        WinSendMsg(WinWindowFromID(hwnd,IDSB_LUN),SPBM_QUERYVALUE,MPFROMP(&ulFlags),
        MPFROM2SHORT(0,SPBQ_ALWAYSUPDATE));
        iLun=ulFlags;
        */
      
      WinSendMsg(WinWindowFromID(hwnd,IDSB_SPEED),SPBM_QUERYVALUE,MPFROMP(&ulFlags),
                 MPFROM2SHORT(0,SPBQ_ALWAYSUPDATE));
      iSpeed=(int)ulFlags;
      
      WinSendMsg(WinWindowFromID(hwnd,IDSB_FIFO),SPBM_QUERYVALUE,MPFROMP(&ulFlags),
                 MPFROM2SHORT(0,SPBQ_ALWAYSUPDATE));
      iFifo=(int)ulFlags;
      
      ulFlags=0;
      if(WinQueryButtonCheckstate(hwnd,IDCB_CDRSTARTHIDDEN)&1)
        ulFlags|=IDCDR_HIDEWINDOW;
      if(WinQueryButtonCheckstate(hwnd,IDCB_CDRCLOSEWINDOW)&1)
        ulFlags|=IDCDR_CLOSEWINDOW;
      lCDROptions=ulFlags;
      /* Let the WPS save the new data */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_SCAN),QWL_USER);
      if(thisPtr)
        thisPtr->wpSaveImmediate();      
      /* Setup is done */
      return (MRESULT) TRUE;
    case WM_COMMAND:    
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_WIDERRUFEN:
          /* User pressed the UNDO button */
          /* WinSendMsg(WinWindowFromID(hwnd,IDSB_BUS),SPBM_SETCURRENTVALUE,MPFROMLONG((LONG)iBus),MPFROMLONG(0));
             WinSendMsg(WinWindowFromID(hwnd,IDSB_TARGET),SPBM_SETCURRENTVALUE,MPFROMLONG((LONG)iTarget),MPFROMLONG(0));
             WinSendMsg(WinWindowFromID(hwnd,IDSB_LUN),SPBM_SETCURRENTVALUE,MPFROMLONG((LONG)iLun),MPFROMLONG(0));*/
          WinSendMsg(WinWindowFromID(hwnd,IDSB_SPEED),SPBM_SETCURRENTVALUE,MPFROMLONG((LONG)iSpeed),MPFROMLONG(0));
          WinSendMsg(WinWindowFromID(hwnd,IDSB_FIFO),SPBM_SETCURRENTVALUE,MPFROMLONG((LONG)iFifo),MPFROMLONG(0));
          if(lCDROptions&IDCDR_HIDEWINDOW)
            WinCheckButton(hwnd,IDCB_CDRSTARTHIDDEN,1);
          if(lCDROptions&IDCDR_CLOSEWINDOW)
            WinCheckButton(hwnd,IDCB_CDRCLOSEWINDOW,1);
          WinSetWindowText(WinWindowFromID(hwnd,IDDD_DEVICESELECTION),chrDeviceName);
          break;
        case IDPB_SCAN:
          _beginthread(scanbusThreadFunc,NULL,8192,(void*)hwnd);

          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}


/***********************************************************************/
/*                                                                     */
/* This procedure handles the cdrdao settings page for audio folders   */
/*                                                                     */
/***********************************************************************/
MRESULT EXPENTRY audioCdrdaoOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  FILEDLG fd = { 0 };
  CWAudioFolder* thisPtr;
  ULONG ulFlags;
  int a;
  char chrCD[4];
  char driveID[40];
  char text[CCHMAXPATH];

  char *ptr;
  HINI hini;
  ULONG ulSize;
  char *pBuffer;
  BOOL bFirst;
  //  HOBJECT  hObject;

  switch(msg)
    {
    case WM_INITDLG :   
      WinSetWindowULong(WinWindowFromID(hwnd,IDPB_CDRDAOBROWSE),
                        QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      WinSendMsg(WinWindowFromID(hwnd,IDEF_CDRDAOPATH),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)CCHMAXPATH),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRDAOPATH),chrCdrdaoPath);
      WinSetWindowText(WinWindowFromID(hwnd,IDLB_CDRDAODRIVER), chrWriterName);
      sprintf(text,"%s\\cfgdata.ini",chrInstallDir);
      hini=PrfOpenProfile(WinQueryAnchorBlock(HWND_DESKTOP),text);
      if(hini)
        {
          if(PrfQueryProfileSize(hini, NULL,NULL,&ulSize)) {
            pBuffer=(char*)malloc(ulSize);
            if(pBuffer) {
              if(PrfQueryProfileData(hini, NULL,NULL, pBuffer, &ulSize))
                {
                  ptr=pBuffer;
                  if(strlen(chrWriterName)==0)
                    bFirst=TRUE;
                  else
                    bFirst=FALSE;
                  do {
                    PrfQueryProfileString(hini,ptr,"driver","",text,sizeof(text));
                    if(strlen(text)!=0) {
                      WinSendMsg(WinWindowFromID(hwnd,IDLB_CDRDAODRIVER),LM_INSERTITEM,MPFROMSHORT(LIT_SORTASCENDING),
                                 (MPARAM)ptr);
                      if(bFirst) {
                        bFirst=FALSE;
                        WinSetWindowText(WinWindowFromID(hwnd,IDLB_CDRDAODRIVER),ptr);
                      }
                    }
                    ptr=strchr(ptr,0);
                    if(!ptr)
                      break;
                    ptr++;
                  }while(*ptr!=0);
                }
              free(pBuffer);
            }
          }
          PrfCloseProfile(hini);
        }
      return (MRESULT) TRUE;
    case WM_HELP:
      break;
    case WM_DESTROY:
      /* The notebook closes and gets destroyed */
      /* Query the cdrdao path from the entryfield */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_CDRDAOPATH),sizeof(chrCdrdaoPath),chrCdrdaoPath);
      /* Query the cdrdao driver from the entryfield for audio folders */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDLB_CDRDAODRIVER ),sizeof(chrWriterName),chrWriterName);
      sprintf(text,"%s\\cfgdata.ini",chrInstallDir);
      hini=PrfOpenProfile(WinQueryAnchorBlock(HWND_DESKTOP),text);
      if(hini)
        {
          PrfQueryProfileString(hini,chrWriterName,"driver","",chrCdrdaoDriver,sizeof(chrCdrdaoDriver));
          PrfCloseProfile(hini);
        }          
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_CDRECORDBROWSE),QWL_USER);
      if(thisPtr)
        thisPtr->wpSaveImmediate();      
      /* Setup is done */
      return (MRESULT) TRUE;
    case WM_COMMAND:    
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_CDRDAOBROWSE:
          /* User pressed the browse button */
          fd.cbSize = sizeof( fd );
          /* It's an centered 'Open'-dialog */
          fd.fl = FDS_OPEN_DIALOG|FDS_CENTER;
          /* Title: "Search CDRecord/2" */
          getMessage(text,IDSTR_FDLGSEARCHCDRDAOTITLE,sizeof(text), hAudioResource,hwnd);
          /* Set the title of the file dialog */
          fd.pszTitle = text;
          /* Only show *.exe files */
          sprintf(fd.szFullFile,"%s","*.exe");

          if( WinFileDlg( HWND_DESKTOP, hwnd, &fd ) == NULLHANDLE )
            {
              /* WinFileDlg failed */
              break;
            }
          if( fd.lReturn == DID_OK )
            {
              WinSetWindowText( WinWindowFromID(hwnd,IDEF_CDRDAOPATH), fd.szFullFile );
            }
          break;
        case IDPB_WIDERRUFEN:
          /* User pressed the UNDO button */
          WinSetWindowText(WinWindowFromID(hwnd,IDLB_CDRDAODRIVER),chrWriterName);
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_CDRDAOPATH),chrCdrdaoPath);
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}


/****************************************************/
/*                                                  */ 
/* This procedure handles the grabber settings page */
/* in the settings notebook                         */
/*                                                  */
/****************************************************/
MRESULT EXPENTRY grabberOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  FILEDLG fd = { 0 };
  CWAudioFolder* thisPtr;
  int a;
  char chrCD[4];
  char text[CCHMAXPATH]; 
 
  switch(msg)
    {
    case WM_INITDLG :	
      WinSetWindowULong(WinWindowFromID(hwnd,IDPB_GRABBERBROWSE),
                        QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      thisPtr=(CWAudioFolder*)PVOIDFROMMP(mp2);
      WinSendMsg(WinWindowFromID(hwnd,IDEF_GRABBERPATH),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)sizeof(chrGrabberPath)),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_GRABBERPATH),chrGrabberPath);
      WinSendMsg(WinWindowFromID(hwnd,IDEF_GRABBEROPTIONS),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)CCHMAXPATH),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_GRABBEROPTIONS),chrGrabberOptions);
      WinCheckButton(hwnd,IDCB_TRACKNUMBERS,bTrackNumbers);
      for( a=0; a<iNumCD ;a++) {
        chrCD[0]=cFirstCD+a;
        chrCD[1]=':';
        chrCD[2]=0;
        WinSendMsg(WinWindowFromID(hwnd,IDLB_DROPDOWN),LM_INSERTITEM,MPFROMSHORT(LIT_END),(MPARAM)chrCD);
        if(!a){
          if(!GrabberSetupDone)
            WinSetWindowText(WinWindowFromID(hwnd,IDLB_DROPDOWN),chrCD);
          else
            WinSetWindowText(WinWindowFromID(hwnd,IDLB_DROPDOWN),chosenCD);
        }
      }
      
      return (MRESULT) TRUE;
    case WM_HELP:	
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_GRABBERBROWSE),QWL_USER);
      if(!thisPtr)
        break;
      switch(WinQueryWindowUShort(WinQueryFocus(HWND_DESKTOP),QWS_ID))
        {
        case IDEF_GRABBEROPTIONS:
          return (MRESULT)thisPtr->wpDisplayHelp(IDEF_GRABBEROPTIONS,AFHELPLIBRARY);
        case IDPB_GRABBERUNDO:
          return (MRESULT)thisPtr->wpDisplayHelp(IDPB_GRABBERUNDO,AFHELPLIBRARY);
        case IDPB_GRABBERBROWSE:
          return (MRESULT)thisPtr->wpDisplayHelp(IDPB_GRABBERBROWSE,AFHELPLIBRARY);
        case IDEF_GRABBERPATH:
          return (MRESULT)thisPtr->wpDisplayHelp(IDEF_GRABBERPATH,AFHELPLIBRARY);
        case IDCB_TRACKNUMBERS:
          return (MRESULT)thisPtr->wpDisplayHelp(IDCB_TRACKNUMBERS,AFHELPLIBRARY);
          
        default:
          break;
        }
      return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_GRABBERSETUP,AFHELPLIBRARY);
      break;
    case WM_DESTROY:
      /* The notebook closes and gets destroyed */
      /* Query the grabber path from the entryfield */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_GRABBERPATH),sizeof(chrGrabberPath),chrGrabberPath);
      /* Query the grabber options from the entryfield */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_GRABBEROPTIONS),sizeof(chrGrabberOptions),chrGrabberOptions);
      /* Query the cd-drive */
      WinQueryWindowText( WinWindowFromID(hwnd,IDLB_DROPDOWN),sizeof(chosenCD),chosenCD);
      /* Let the WPS save the new instance data */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_GRABBERBROWSE),QWL_USER);
      bTrackNumbers=WinQueryButtonCheckstate(hwnd,IDCB_TRACKNUMBERS);
      if(thisPtr) {
        thisPtr->wpSaveDeferred();
      }
      /* Setup is done */
      GrabberSetupDone=TRUE;
      return (MRESULT) TRUE;
    case WM_COMMAND:	
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_GRABBERBROWSE:
          /* User pressed the browse button */
          fd.cbSize = sizeof( fd );
          /* It's an centered 'Open'-dialog */
          fd.fl = FDS_OPEN_DIALOG|FDS_CENTER;
          /* Set the title of the file dialog */
          /* Title: "Search Audio-Grabber" */
          getMessage(text,IDSTR_FDLGSEARCHGRABBERTITLE,sizeof(text), hAudioResource,hwnd);
          fd.pszTitle = text;
          /* Only show *.exe files */
          sprintf(fd.szFullFile,"%s","*.exe");
          
          if( WinFileDlg( HWND_DESKTOP, hwnd, &fd ) == NULLHANDLE )
            {
              /* WinFileDlg failed */
              break;
            }
          if( fd.lReturn == DID_OK )
            {
              WinSetWindowText( WinWindowFromID(hwnd,IDEF_GRABBERPATH), fd.szFullFile );
            }
          break;
        case IDPB_GRABBERUNDO:
          /* User pressed the UNDO button */
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_GRABBERPATH),chrGrabberPath);
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_GRABBEROPTIONS),chrGrabberOptions);
          WinSetWindowText(WinWindowFromID(hwnd,IDLB_DROPDOWN),chosenCD);
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}

/****************************************************/
/*                                                  */ 
/* This procedure handles the mpg123 settings page  */
/* in the settings notebook                         */
/*                                                  */
/****************************************************/
MRESULT EXPENTRY mpg123OptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) 
{
  FILEDLG fd = { 0 };
  CWAudioFolder* thisPtr;
  int a;
  char chrCD[4];
  char text[CCHMAXPATH]; 
 
  switch(msg)
    {
    case WM_INITDLG :	
      WinSetWindowULong(WinWindowFromID(hwnd,IDPB_MPG123BROWSE),
                        QWL_USER,(ULONG)PVOIDFROMMP(mp2));//Save object ptr.
      thisPtr=(CWAudioFolder*)PVOIDFROMMP(mp2);
      WinSendMsg(WinWindowFromID(hwnd,IDEF_MPG123PATH),EM_SETTEXTLIMIT,MPFROMSHORT((SHORT)sizeof(chrMpg123Path)),0);
      WinSetWindowText(WinWindowFromID(hwnd,IDEF_MPG123PATH),chrMpg123Path);
      if(bMpg123SwabBytes)
        WinCheckButton(hwnd,IDCB_MPG123SWABBYTES,1);
      else
        WinCheckButton(hwnd,IDCB_MPG123SWABBYTES,0);
      return (MRESULT) TRUE;
    case WM_HELP:	
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_GRABBERBROWSE),QWL_USER);
      if(!thisPtr)
        break;
      switch(WinQueryWindowUShort(WinQueryFocus(HWND_DESKTOP),QWS_ID))
        {
          /*        case IDPB_MPG123UNDO:
                    return (MRESULT)thisPtr->wpDisplayHelp(IDPB_GRABBERUNDO,AFHELPLIBRARY);
                    case IDPB_MPG123BROWSE:
                    return (MRESULT)thisPtr->wpDisplayHelp(IDPB_GRABBERBROWSE,AFHELPLIBRARY);
                    case IDEF_MPG123PATH:
                    return (MRESULT)thisPtr->wpDisplayHelp(IDEF_GRABBERPATH,AFHELPLIBRARY);*/
        default:
          break;
        }
      //return (MRESULT)thisPtr->wpDisplayHelp(IDDLG_GRABBERSETUP,AFHELPLIBRARY);
      break;
    case WM_DESTROY:
      /* The notebook closes and gets destroyed */
      /* Query the mpg123 path from the entryfield */     
      WinQueryWindowText( WinWindowFromID(hwnd,IDEF_MPG123PATH),sizeof(chrMpg123Path),chrMpg123Path);
      if(WinQueryButtonCheckstate(hwnd,IDCB_MPG123SWABBYTES)&1)
        bMpg123SwabBytes=TRUE;
      else
        bMpg123SwabBytes=FALSE;
      /* Let the WPS save the new instance data */
      thisPtr=(CWAudioFolder*) WinQueryWindowULong(WinWindowFromID(hwnd,IDPB_MPG123BROWSE),QWL_USER);
      if(thisPtr) {
        thisPtr->wpSaveImmediate();
      }
      /* Setup is done */
      return (MRESULT) TRUE;
    case WM_COMMAND:	
      switch(SHORT1FROMMP(mp1))
        {
        case IDPB_MPG123BROWSE:
          /* User pressed the browse button */
          fd.cbSize = sizeof( fd );
          /* It's an centered 'Open'-dialog */
          fd.fl = FDS_OPEN_DIALOG|FDS_CENTER;
          /* Set the title of the file dialog */
          /* Title: "Search Audio-Grabber" */
          getMessage(text, IDSTR_FDLGSEARCHMPG123TITLE, sizeof(text), hAudioResource,hwnd);
          fd.pszTitle = text;
          /* Only show *.exe files */
          sprintf(fd.szFullFile,"%s","*.exe");
          
          if( WinFileDlg( HWND_DESKTOP, hwnd, &fd ) == NULLHANDLE )
            {
              /* WinFileDlg failed */
              break;
            }
          if( fd.lReturn == DID_OK )
            {
              WinSetWindowText( WinWindowFromID(hwnd,IDEF_MPG123PATH), fd.szFullFile );
            }
          break;
        case IDPB_MPG123UNDO:
          /* User pressed the UNDO button */
          WinSetWindowText(WinWindowFromID(hwnd,IDEF_MPG123PATH),chrMpg123Path);
          if(bMpg123SwabBytes)
            WinCheckButton(hwnd,IDCB_MPG123SWABBYTES,1);
          else
            WinCheckButton(hwnd,IDCB_MPG123SWABBYTES,0);
          break;
        default:
          break;
        }
      return (MRESULT) TRUE;
    default:
      break;
    }
  return WinDefDlgProc(hwnd, msg, mp1, mp2);
}









