/*
 * This file is (C) Chris Wohlgemuth 1999
 * 
 * It's part of the Audio/Data-CD-Creator distribution
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define INCL_DOSQUEUES
#include "audiofolder.hh"
#include "audiofolderhelp.h"
#include <stdio.h>
#include <stdlib.h>
#include <wpshadow.hh>
#include <string.h>



extern PVOID pvAudioSharedMem;
extern BOOL setupDone;
extern BOOL GrabberSetupDone;

extern int iNumCD;
extern char cFirstCD;
extern char chosenCD[3];
extern char chosenWriter[3];// Drive letter of CD-writer

extern char chrGrabberPath[CCHMAXPATH];
extern char chrGrabberOptions[CCHMAXPATH];
extern int bTrackNumbers;
extern LONG lCDROptions;

extern char chrCDRecord[CCHMAXPATH];/* Path to cdrecord */
extern char chrDataCDROptions[CCHMAXPATH];
extern char chrInstallDir[CCHMAXPATH];

extern char chrMpg123Path[CCHMAXPATH];

extern ATOM atomUpdateStatusbar;
extern ATOM atomStartGrab;
extern ATOM atomStartWrite;

extern HMODULE hAudioResource;

typedef struct
{
  USHORT cb;
  CWAudioFolder* thisPtr;
}WCNTRLDATA;


HFILE openDrive(char * drive);
void closeDrive(HFILE hfDrive);
ULONG queryAudioCDTracks(HFILE hfDrive);

MRESULT EXPENTRY CDToolsDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY trackDialogProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
ULONG messageBox( char* text, ULONG ulTextID , LONG lSizeText,
                  char* title, ULONG ulTitleID, LONG lSizeTitle,
                  HMODULE hResource, HWND hwnd, ULONG ulFlags);
void getMessage(char* text,ULONG ulID, LONG lSizeText, HMODULE hResource,HWND hwnd);
void sendConfig();
ULONG launchWrapper(PSZ parameter, PSZ folderPath,HWND hwnd,PSZ wrapperExe, PSZ pszTitle);
PSZ buildAudioWriteParam(CWAudioFolder* thisPtr, PSZ trackname);
PSZ buildWrapperName();
ULONG launchGrabber(PSZ parameter, PSZ folderPath,HWND hwnd);
void changeBackslash(char* text);

/* Dialog procs for the settings pages */
MRESULT EXPENTRY grabberOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY audioCdrecordOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY mpg123OptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY audioCdrdaoOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;
MRESULT EXPENTRY generalOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) ;

void errorResource();

/* Local */
HMODULE queryModuleHandle(void);
PSZ buildGrabberParam(int numTrack,PSZ trackname);


/* This function returns the module handle of our class-dll */
HMODULE queryModuleHandle()
{
  static HMODULE hmod=0;
    
  if(!hmod) {
    PSZ pathname=SOMClassMgrObject  //Query Pathname of class file
      ->somLocateClassFile(somIdFromString("CWAudioFolder"),1,2);
    DosQueryModuleHandle(pathname,&hmod);  //Query module handle
  }
  return hmod;
}

void CWAudioFolder::cwWriteAudioTracks(HWND hwnd, char* trackName, char* folderName)
{
  char *text;
  char logFilePath[CCHMAXPATH];
  
  if(!pvAudioSharedMem) {
    if(DosAllocSharedMem(&pvAudioSharedMem,"\\SHAREMEM\\AUDIOCMDLINE",SHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
      WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, "Can't allocate shared Mem!","Write audio tracks",
                      0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
      return;
    }
  }
  memset(pvAudioSharedMem,0,AUDIOSHAREDMEM_SIZE);
  text= (char*)pvAudioSharedMem;

  /* Put CDRecord/2 path into cmd line */
  sprintf(text,"%s ",chrCDRecord);             

  /* Add audio options and trackname */
  strcat(text,buildAudioWriteParam(this,trackName));           

  sprintf(logFilePath,"\"%s\"",folderName);
  launchWrapper("", logFilePath, hwnd,"writeaud.exe", "CDRecord/2");  
  return;
}

/******************************************************/
/* This function checks if the object is a wave file. */
/* It only checks the extension because cdrecord      */
/* doesn't know types and needs the extension to      */
/* distinguish between wavefiles and raw data files.  */
/******************************************************/
BOOL CWAudioFolder::cwIsWaveFile(WPObject* wpObject)
{
  char objectName[CCHMAXPATH];
  ULONG ulNameSize;
  char* chrExtension;

  if(!somResolveByName(wpObject,"wpQueryRealName"))
    /* It's not a file system object */
    return FALSE;

  /* Check if it's folder */
  if(somResolveByName(wpObject,"wpQueryContent"))
    /* It's a folder */
    return FALSE;

  /* Check the extension */
  ulNameSize=sizeof(objectName);
  ((WPFileSystem*)wpObject)->wpQueryRealName(objectName,&ulNameSize,FALSE);
  chrExtension=strstr(strlwr(objectName),".wav");
  if(chrExtension)
    if(*(chrExtension+4)==0)return TRUE;//It's a wavefile
  
  return FALSE;
}

/******************************************************/
/* This function checks if the object is a mp3 file.  */
/* It only checks the extension.                      */
/******************************************************/
BOOL CWAudioFolder::cwIsMp3File(WPObject* wpObject)
{
  char objectName[CCHMAXPATH];
  ULONG ulNameSize;
  char* chrExtension;

  if(!somResolveByName(wpObject,"wpQueryRealName"))
    /* It's not a file system object */
    return FALSE;

  /* Check if it's folder */
  if(somResolveByName(wpObject,"wpQueryContent"))
    /* It's a folder */
    return FALSE;

  /* Check the extension */
  ulNameSize=sizeof(objectName);
  ((WPFileSystem*)wpObject)->wpQueryRealName(objectName,&ulNameSize,FALSE);
  chrExtension=strstr(strlwr(objectName),".mp3");
  if(chrExtension)
    if(*(chrExtension+4)==0)return TRUE;//It's a mp3file
  
  return FALSE;
}


/* This thread handles the CDTools-dialog */
void _Optlink toolsThreadFunc (void *arg)
{
  HWND hwndTools;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;
  CWAudioFolder *thisPtr;
  
  thisPtr=(CWAudioFolder*)arg;    //Pointer auf CWAudioFolder-Object
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {
      hwndTools=WinLoadDlg(HWND_DESKTOP,HWND_DESKTOP,CDToolsDlgProc,hAudioResource,IDDLG_CDRTOOLS,thisPtr);
      if(hwndTools) {
        /* CDR tools window created */
        while(WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))
          WinDispatchMsg(hab,&qmsg);
        WinDestroyWindow(hwndTools);
      }
      WinDestroyMsgQueue(hmq);
    }
    WinTerminate(hab);
  }
}


/********************************************************/
/* New class function which inserts the Cdrecord option */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWAudioFolder::cwAddGeneralOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
    char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = generalOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hAudioResource;
  //pageinfo.resid = queryModuleHandle();
  //The ID of the dialog template
  pageinfo.dlgid = IDDLG_GENERAL;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_GENERAL;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "General" */
  getMessage(pageName, IDSTRSETTING_GENERAL, sizeof(pageName),  hAudioResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}


/********************************************************/
/* New class function which inserts the Cdrecord option */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWAudioFolder::AddCdrecordOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
    char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = audioCdrecordOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hAudioResource;
  //pageinfo.resid = queryModuleHandle();
  //The ID of the dialog template
  pageinfo.dlgid = IDDLG_CDRECORDSETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_CDRECORDSETUP;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "Helper applications" */
  getMessage(pageName, IDSTRSETTING_HELPERAPPLICATIONS, sizeof(pageName),  hAudioResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}


/********************************************************/
/* New class function which inserts the cdrdao          */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWAudioFolder::AddCdrdaoOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = audioCdrdaoOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hAudioResource;
  //pageinfo.resid = queryModuleHandle();
  //The ID of the dialog template
  pageinfo.dlgid = IDDLG_CDRDAOSETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_CDRDAOSETUP;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

  
/********************************************************/
/* New class function which inserts the Grabber option  */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWAudioFolder::AddGrabOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  
  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags =  BKA_STATUSTEXTON;
  //pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = grabberOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hAudioResource;
  //pageinfo.resid = queryModuleHandle();
  pageinfo.dlgid = IDDLG_GRABBERSETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_GRABBERSETUP;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  //pageinfo.pszName = "Grabber options";
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the mpg123 option   */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWAudioFolder::AddMpg123OptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  
  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags =  BKA_STATUSTEXTON;
  //pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = mpg123OptionDlgProc;
  //The resource DLL
  pageinfo.resid = hAudioResource;
  //pageinfo.resid = queryModuleHandle();
  pageinfo.dlgid = IDDLG_MPG123SETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_MPG123SETUP;
  //Tell the WPS the help library name
  //pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  //pageinfo.pszName = "Grabber options";
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}


void CWAudioFolder::cwSetStatusTime(ULONG ulTrackSize)
{
  char text[200];
  ULONG ulSize;
    
  ulSize=ulTrackSize;
  ulSize/=(2*2*44100);/* Calculate seconds */
  sprintf(text,"Total time: %d min %02d sec",ulSize/60 , (ulSize % 60));
  WinSetWindowText(WinWindowFromID(hwndStatusFrameCtl,IDST_STATUSTOTALTIME),text);    
}

ULONG CWAudioFolder::cwQueryWriteFlags()
{
  return ulWriteFlags;
}

ULONG CWAudioFolder::cwSetWriteFlags(ULONG ulNewWriteFlags,ULONG ulMask)
{
  ULONG rc;

  rc=cwQueryWriteFlags();
  ulWriteFlags=((ulNewWriteFlags&ulMask) | (~ulMask & ulWriteFlags));

  return rc;/* return previous flags */
}

void CWAudioFolder::cwEnableGrab(BOOL bEnable)
{
  bGrabEnabled=bEnable;
}

BOOL CWAudioFolder::cwQueryGrabEnabled(void)
{   
  return bGrabEnabled;    
}

void CWAudioFolder::cwEnableWrite(BOOL bEnable)
{
  bWriteEnabled=bEnable;
}

BOOL CWAudioFolder::cwQueryWriteEnabled(void)
{   
  return bWriteEnabled;    
}

void CWAudioFolder::cwEnableGrabControls(BOOL bEnable)
{
  if(!hwndGrab)
    return;
  WinEnableWindow(WinWindowFromID(hwndGrab,IDPB_SELECTALL) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndGrab,IDLB_GRABTRACKS) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndGrab,IDPB_GRAB) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndGrab,IDPB_REFRESH) ,bEnable);
    
}

void CWAudioFolder::cwEnableSelectControls(BOOL bEnable)
{
  if(!hwndSelect)
    return;
  WinEnableWindow(WinWindowFromID(hwndSelect,IDRB_WRITE) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndSelect,IDRB_GRAB) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndSelect,IDEF_TRACKNAME) ,bEnable);
}

void CWAudioFolder::cwForceStatusUpdate(void)
{
  if(atomUpdateStatusbar)
    WinPostMsg(hwndStatusFrameCtl,atomUpdateStatusbar,0,MPFROMP(this));     
}

void CWAudioFolder::cwEnableWriteControls(BOOL bEnable)
{
  if(!hwndWrite)
    return;
  WinEnableWindow(WinWindowFromID(hwndWrite,IDCB_PAD) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndWrite,IDCB_NOFIX) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndWrite,IDCB_PREEMP) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndWrite,IDCB_DUMMY) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndWrite,IDCB_DAO) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndWrite,IDPB_BURN) ,bEnable);
}

void CWAudioFolder::cwSetStatusText(char * text)
{ 
  WinSetWindowText(WinWindowFromID(hwndStatusFrameCtl,IDST_STATUSTOTALTIME),text);    
}

/*********************************************************************/
/*                                                                   */
/* This procedure fills the given listbox with all the tracks of the */
/* given folder. If no listbox is given only the size of all waves   */
/* is returned. Mp3 are currently ignored!                           */
/*                                                                   */
/* Parameters:                                                       */
/*              hwndFrame:   frame handle of wpFolder window         */
/*              hwndListBox: handle of listbox where to insert the   */
/*                           wavefile names                          */
/*                                                                   */
/* Returns: Size of all wave tracks in byte                          */
/*                                                                   */
/*********************************************************************/
ULONG CWAudioFolder::cwFillTrackList(HWND hwndFrame, HWND hwndListBox)
{
  HWND hwndCnr;
  PMINIRECORDCORE mrc;   
  WPObject * contentObject;
  char name[CCHMAXPATH];
  ULONG ulNameSize;
  ULONG ulSize;
  BOOL bIsMp3;

  ulSize=0;
  /* Get hwnd of folder container */ 
  hwndCnr=WinWindowFromID(hwndFrame,FID_CLIENT);
  if(hwndCnr){ /* Catch error */
    /* Get first container item of our folder */
    mrc=(PMINIRECORDCORE)WinSendMsg(hwndCnr,CM_QUERYRECORD,NULL,
                                    MPFROM2SHORT(CMA_FIRST,CMA_ITEMORDER));
    if(mrc){ 
      while(mrc) {
        /* Get wps-object-ptr. from container item */
        contentObject=(WPObject*)OBJECT_FROM_PREC(mrc);//Get object
        ulNameSize=sizeof(name);
        if(contentObject) {
          /* Get file system object or NULL */
          contentObject=cwGetFileSystemObject(contentObject);
        }
        if(contentObject){
          /* It's a file system object */
          /* Check, if it's a wave file or a mp3file */
          bIsMp3=cwIsMp3File(contentObject);
          if(cwIsWaveFile(contentObject)|| (bIsMp3 && strlen(chrMpg123Path) && !(ulWriteFlags & IDWF_DAO))){
            /* Yes, query the full path */
            ((WPFileSystem*)contentObject)->wpQueryRealName(name,&ulNameSize,TRUE);
            /* Add tracksize of wavefile to totalsize */ 
            if(!bIsMp3)          
              ulSize+=((WPFileSystem*)contentObject)->wpQueryFileSize()-44;
            else {
              /* Add playtime of mp3file to totalsize */ 
            }
            /* Add 2 seconds for pause between tracks */
            //ulSize+=8*44100;
            /* insert the name into the listbox */ 
            if(hwndListBox)
              WinInsertLboxItem(hwndListBox,LIT_END,
                                name);
            
          }           
        }/* end of if(contentObject) */           
        /* Get next container item */
        mrc=(PMINIRECORDCORE)WinSendMsg(hwndCnr,CM_QUERYRECORD,MPFROMP(mrc),
                                        MPFROM2SHORT(CMA_NEXT,CMA_ITEMORDER));
      }// end of while(mrc)
    }// end of if(mrc)
  }
  return ulSize;  
}

/*********************************************************************/
/*             hwndListBox: handle of listbox holding the tracknames */
/*********************************************************************/
BOOL buildTocFile( CWAudioFolder *thisPtr, HWND hwndListBox)
{
  char name[CCHMAXPATH];
  ULONG ulNameSize;
  ULONG ulSize;
  int b;
  LONG a;
  FILE *fStream;
  ULONG ulFlags;
  BOOL bFirstTrack;

  /* Query the full path of our folder*/
  ulNameSize=sizeof(name);
  thisPtr->wpQueryRealName(name,&ulNameSize,TRUE);
  strcat(name,"\\default.toc");
  fStream=fopen(name,"w");
  if(!fStream)
    return FALSE;

  bFirstTrack=TRUE;
  ulFlags=thisPtr->cwQueryWriteFlags();
  /* Build toc file */
  fprintf(fStream,"CD_DA\n\n");
  a=SHORT1FROMMR(WinSendMsg(hwndListBox,LM_QUERYITEMCOUNT, NULL, NULL));
  for(b=0;b<a;b++) {
    if(SHORT1FROMMR(WinSendMsg(hwndListBox,LM_QUERYITEMTEXT,MPFROM2SHORT((SHORT)b,(SHORT)CCHMAXPATH),&name))) {
      changeBackslash(name);        
      fprintf(fStream,"//Track %d\n",b+1);
      fprintf(fStream,"TRACK AUDIO\n");
      if(ulFlags & IDWF_PREEMP)
        fprintf(fStream,"PRE_EMPHASIS\n");
      /* Pregap */
      /*      if(bFirstTrack) {
              fprintf(fStream,"PREGAP 0:2:0\n");
              bFirstTrack=FALSE;
              }*/
      fprintf(fStream,"FILE \"%s\" 0\n\n", name);
    }
  }
  fclose(fStream);
  return TRUE;
}

























