/*
 * This file is (C) Chris Wohlgemuth 1999
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#define INCL_DOSERRORS

#include "audiofolder.hh"
#include "audiofolderhelp.h"
#include <stdio.h>
#include <string.h>

extern HMODULE hAudioResource;
extern ATOM atomUpdateStatusbar;
extern ATOM atomStartGrab;
extern ATOM atomStartWrite;

HMODULE queryModuleHandle(void);

/* Overriden function which returns our class name */
PSZ M_CWAudioFolder::wpclsQueryTitle()
{
  return "Audio-CD-Creator";
}

/* Overriden function which returns the custom icon */
ULONG M_CWAudioFolder::wpclsQueryIconData(PICONINFO pIconInfo)
{
  if (pIconInfo)   {
    pIconInfo->fFormat = ICON_RESOURCE;
    /* The function queryModuleHandle() returns the module handle
       of the DLL the class resides in. The resource compiler put
       the icon in the DLL after compiling the class */ 
    //pIconInfo->hmod    = queryModuleHandle();
    pIconInfo->hmod    = hAudioResource;
    pIconInfo->resid   = ID_ICONAUDIOFOLDER;
  } /* endif */
  return ( sizeof(ICONINFO) );
}

ULONG M_CWAudioFolder::wpclsQueryIconDataN(PICONINFO pIconInfo, ULONG ulIconIndex)
{

  if (pIconInfo)   {
    pIconInfo->fFormat = ICON_RESOURCE;
    /* The function queryModuleHandle() returns the module handle
       of the DLL the class resides in. The resource compiler put
       the icon in the DLL after compiling the class */ 
    //pIconInfo->hmod    = queryModuleHandle();
    pIconInfo->hmod    = hAudioResource;
    pIconInfo->resid   = ID_ICONAUDIOFOLDER2;
  } /* endif */
  return ( sizeof(ICONINFO) );
}

ULONG M_CWAudioFolder::wpclsQueryStyle()
{
  /* CWProgFolder returns *_NEVERTEMPLATE but we want a template */
  return M_WPFolder::wpclsQueryStyle();
}

BOOL M_CWAudioFolder::wpclsQueryDefaultHelp(PULONG HelpPanelId,PSZ HelpLibrary)
{
	if(HelpLibrary)
      /* The name of the library is defined in audiofolderhelp.h */
		strcpy(HelpLibrary,AFHELPLIBRARY);
	if(HelpPanelId)
		*HelpPanelId= IDHLP_MAIN;
	
	return TRUE;
}

void M_CWAudioFolder::wpclsInitData()
{
  COUNTRYCODE country= {0};
  COUNTRYINFO countryInfo= {0};
  ULONG ulInfoLen=0;
  APIRET rc;
  char path[CCHMAXPATH];  
  char* found;
  char buf[CCHMAXPATH];
  HATOMTBL hatSystem;

  /* Call parent */
  M_CWProgFolder::wpclsInitData();

  /* Get unique window messages for our internal communication */
  hatSystem=WinQuerySystemAtomTable();
  /* Window message for updating the statusbar */
  atomUpdateStatusbar=WinAddAtom(hatSystem,WM_UPDATESTATUSBARMSG);
  /* Window message for starting a grab  */
  atomStartGrab=WinAddAtom(hatSystem,WM_STARTGRABMSG);
  /* Window message for starting a write */
  atomStartWrite=WinAddAtom(hatSystem,WM_STARTWRITEMSG);


  /* Get the country code of our system and load the  
     resource DLL with the right language */
  do {
    rc=DosQueryCtryInfo(sizeof(countryInfo),&country,&countryInfo,&ulInfoLen);
    if(rc!=NO_ERROR)
      break;
    
    rc=DosQueryModuleName(queryModuleHandle(),sizeof(path),path);
    if(rc!=NO_ERROR)
      break;

    found=strrchr(path,'\\');
    if(!found)
      break;

    *found=0;
    sprintf(buf,"\\cdfld%03d.dll",countryInfo.country);
    strcat(path,buf);
    //WinMessageBox(  HWND_DESKTOP,0, path, "Search DLL...", 0UL, MB_YESNO | MB_ICONEXCLAMATION|MB_MOVEABLE );
    rc=DosLoadModule(buf,sizeof(buf),path, &hAudioResource);
    if(rc==NO_ERROR)
      break;
    
    /* NLS DLL not found. Try to load default */
    found=strrchr(path,'\\');
    if(!found)
      break;
    
    *found=0;
    sprintf(buf,"\\cdfld001.dll");
    strcat(path,buf);
    rc=DosLoadModule(buf,sizeof(buf),path, &hAudioResource);
    if(rc!=NO_ERROR)
      hAudioResource=NULLHANDLE;
    break;
  }while(TRUE);

}

void M_CWAudioFolder::wpclsUnInitData()
{

  M_CWProgFolder::wpclsUnInitData();
  DosFreeModule(hAudioResource);

}




