/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys\types.h>
#include <sys\stat.h>
#include <stdio.h>

#include "audiofolder.hh"
#include "audiofolderhelp.h"

 
extern PVOID pvSharedMem;

extern char chrImage[CCHMAXPATH];/* Path to iso-image */
extern char chrCDRecord[CCHMAXPATH];/* Path to cdrecord */
extern char chrDataCDROptions[CCHMAXPATH];
extern char chrAudioCDROptions[CCHMAXPATH];
extern char chrGrabberPath[CCHMAXPATH];
extern char chrGrabberOptions[CCHMAXPATH];
extern LONG lCDROptions;
extern char chrInstallDir[CCHMAXPATH];
extern LONG lMKOptions;
extern BOOL bMultiSessionDone;
extern char chosenMultiSessionCD[3];
extern char chrMkisofs[CCHMAXPATH];/* Path to mkisofs */
extern char chrMkisofsOptions[CCHMAXPATH];

extern char chrCdrdaoDriver[100];

extern HMODULE hAudioResource;

extern int iBus;
extern int iTarget;
extern int iLun;
extern int iSpeed;
extern int iFifo;

extern char chosenCD[3];
extern int bTrackNumbers;

HMODULE queryModuleHandle(void);
void changeBackslash(char* text);
void getMessage(char* text,ULONG ulID, LONG lSizeText, HMODULE hResource,HWND hwnd);

PSZ buildWrapName(PSZ wrapperName);
PSZ buildWriteParam(CWAudioFolder* thisPtr, PSZ trackname);
ULONG launchWriter(PSZ parameter, PSZ folderPath,HWND hwnd, PSZ wrapperExe,  PSZ pszTitle);

PSZ buildWrapName(PSZ wrapperName)
{
  char exename[CCHMAXPATH];

  /* Build full path */
  sprintf(exename,"%s\\%s",chrInstallDir,wrapperName);      
  return exename;
}

/* This function builds the audio params for CDRecord/2 */
PSZ buildAudioWriteParam(CWAudioFolder* thisPtr, PSZ trackname)
{	
  char chrParams[CCHMAXPATH*2];
  char *chrPtr;
  ULONG ulWriteFlagsLocal;

  if(thisPtr)
    ulWriteFlagsLocal=thisPtr->cwQueryWriteFlags();
  
  //  sprintf(chrParams,"%s -audio ",chrAudioCDROptions);
  /* Put dev=.., speed and fs= into the cmd line */
  sprintf(chrParams,"dev=%d,%d,%d speed=%d -audio fs=%dm %s ", iBus, iTarget, iLun, iSpeed, iFifo, chrAudioCDROptions);
  chrPtr=chrParams;

  if(ulWriteFlagsLocal & IDWF_DUMMY)
    chrPtr=strcat(chrPtr,"-dummy ");

  /* Make -pad the default for audio tracks */
  chrPtr=strcat(chrPtr,"-pad ");
                      
  /* if(thisPtr->bWrite+1 == thisPtr->aWrite) {*/
  if(ulWriteFlagsLocal & IDWF_NOFIX)
    chrPtr=strcat(chrPtr,"-nofix ");        
  
  if(ulWriteFlagsLocal & IDWF_PREEMP)
    chrPtr=strcat(chrPtr,"-preemp ");

  /*if(ulWriteFlagsLocal & IDWF_DAO)
    chrPtr=strcat(chrPtr,"-dao ");*/

  /* make sure we have a -v so write status works */
  if(!strstr(chrPtr,"-v"))
    strcat(chrPtr,"-v ");
  if(strlen(trackname)) {
    strcat(chrPtr,"\"");
    strcat(chrPtr,trackname);
    strcat(chrPtr,"\"");
  }
  return chrParams;
}

/* This function builds the audio params for cdrdao/2 */
/*                                                    */
/* Currently no support for buffers!!!                */
/*                                                    */
PSZ buildAudioWriteParamDAO(CWAudioFolder* thisPtr)
{	
  char chrParams[CCHMAXPATH*2];
  char *chrPtr;
  ULONG ulWriteFlagsLocal;
  int buffers;  

  if(thisPtr)
    ulWriteFlagsLocal=thisPtr->cwQueryWriteFlags();
  
  buffers=iFifo*1024/176;
  chrPtr=chrParams;
  /* Simulate or real write */
  if(ulWriteFlagsLocal & IDWF_DUMMY)
     sprintf(chrParams,"simulate --device %d,%d,%d --driver %s --speed %d --buffers %d ",
             iBus, iTarget, iLun, chrCdrdaoDriver,iSpeed, buffers);
  else
    sprintf(chrParams,"write --device %d,%d,%d --driver %s --speed %d --buffers %d ",
            iBus,iTarget,iLun,chrCdrdaoDriver,iSpeed, buffers);
 

  /* make sure we have a -v so write status works */

  return chrParams;
}

void buildDataWriteParam(CWDataFolder* thisPtr, char * text)
{	

  ULONG ulWriteFlagsLocal;

  if(thisPtr)
    ulWriteFlagsLocal=thisPtr->cwQueryCDTypeFlags();
  
  /* Build cdrecord/2 cmd-line parameters */
  //  sprintf(text,chrCDROptions);//Cdrecord/2-options
  sprintf(text,"dev=%d,%d,%d speed=%d fs=%dm %s", iBus, iTarget, iLun, iSpeed, iFifo, chrDataCDROptions);
  /* Check if we write a multisession CD */
  if( ulWriteFlagsLocal & IDCDT_MULTISESSION)
    strcat(text," -multi");//multisessionswitch
  else {
    /* We write a singlesession disk so give the right track type */
    if(ulWriteFlagsLocal & IDCDT_TRACKDATA)
      strcat(text," -data");
    if(ulWriteFlagsLocal & IDCDT_TRACKMODE2)
      strcat(text," -mode2");
    if(ulWriteFlagsLocal & IDCDT_TRACKXA1)
      strcat(text," -xa1");
    if(ulWriteFlagsLocal & IDCDT_TRACKXA2)
      strcat(text," -xa2");
    if(ulWriteFlagsLocal & IDCDT_TRACKCDI)
      strcat(text," -cdi");
  }
  /* make sure we have a -v so write status works */
  if(!strstr(text,"-v"))
    strcat(text," -v");

  return;
}

/* The outputname may be NULL. If so the -o switch will be omitted */
BOOL buildMkisofsParam(CWDataFolder* thisPtr, char * text, char * outputName)
{	
  char *chrPtrParam;
  ULONG ulFlags;
  char filename[CCHMAXPATH];
  char chrDev[20];
  char * chrPtr;
  char * chrPtr2;
  ULONG ulSize;
  HWND hwnd;
  PMINIRECORDCORE mrc;   
  WPObject * contentObject;
  LONG lValue1,lValue2;

  /* Build cmd-line for mkisofs */
  ulFlags=thisPtr->cwQueryMkisofsFlags();
  /* Path to mkisofs */
  strcat(text,chrMkisofs);
  /* Mark end of mkisofs path */
  strcat(text,"\"");

  /* Additional options specified by the user on the settings page */
  strncat(text,chrMkisofsOptions,sizeof(chrMkisofsOptions));

  /* Filename options */
  if(ulFlags & IDMK_ALLOW32CHARS)
    strncat(text," -l",4);
  if(ulFlags & IDMK_LEADINGPERIODS)
    strncat(text," -L",4);
  if(ulFlags & IDMK_JOLIET)
    strncat(text," -J",4);
  if(ulFlags & IDMK_ROCKRIDGE)
    strncat(text," -r",4);
  if(ulFlags & IDMK_TRANSTABLE)
    strncat(text," -T",4);
  if(ulFlags & IDMK_ALLFILES)
    strncat(text," -a",4);
  //  if(ulFlags & IDMK_LOGFILE)
  //   strncat(text," -log-filemkisofs.log",sizeof(" -log-filemkisofs.log"));

  /* Add the autor-stuff */
  if(strlen(thisPtr->chrApplication)) {
    strncat(text," \"-A",strlen(" \"-A"));
    strncat(text,thisPtr->chrApplication,strlen(thisPtr->chrApplication));
    strncat(text,"\"",strlen("\""));
  }
  if(strlen(thisPtr->chrPublisher)) {
    strncat(text," \"-P",strlen(" \"-P"));
    strncat(text,thisPtr->chrPublisher,strlen(thisPtr->chrPublisher));
    strncat(text,"\"",strlen("\""));
  }
  if(strlen(thisPtr->chrPreparer)) {
    strncat(text," \"-p",strlen(" \"-p"));
    strncat(text,thisPtr->chrPreparer,strlen(thisPtr->chrPreparer));
    strncat(text,"\"",strlen("\""));
  }
  if(strlen(thisPtr->chrVolumeName)) {
    /* \" not tested here, yet */
    strncat(text," \"-V",strlen(" \"-V"));
    strncat(text,thisPtr->chrVolumeName,strlen(thisPtr->chrVolumeName));
    strncat(text,"\"",strlen("\""));
  }

  /*  if(strlen(chrCopyright)) {
      strncat(text," -copyright",strlen(" -copyright"));
      strncat(text,chrCopyright,strlen(chrCopyright));
      }*/

  ulFlags=thisPtr->cwQueryCDTypeFlags();
  if((ulFlags & IDCDT_MULTISESSION) && !(ulFlags & IDCDT_FIRSTSESSION)) {
    /* We are building the second image of a multisession disk */
    /* Check if we did some setup stuff on the ms-page. Caution! Isn't bullet proof! */
    if(!bMultiSessionDone) {
      sprintf(text,"You have to provide some information on the 'CD-type selection' page prior to the creating \
of multisession CDs!");  
      WinMessageBox(  HWND_DESKTOP, HWND_DESKTOP,
                      text,
                      "Imagefile creation error",
                      0UL, MB_OK | MB_ICONEXCLAMATION |MB_MOVEABLE);
      DosFreeMem(pvSharedMem);
      pvSharedMem=NULL;
      return FALSE;
    }/* if(!bMultiSessionDone) */

    /* Check if we have the necessary sectornumbers */
    lValue1=thisPtr->cwQueryPreviousStartSector();
    lValue2=thisPtr->cwQueryNextStartSector();
    if(lValue1==lValue2) {
      sprintf(text,"Need sector information of previous session. Go to the 'CD-type selection' page in \
the settings notebook to supply it.");  
      WinMessageBox(  HWND_DESKTOP, HWND_DESKTOP,
                      text,
                      "Imagefile creation error",
                      0UL, MB_OK | MB_ICONEXCLAMATION |MB_MOVEABLE);
      DosFreeMem(pvSharedMem);
      pvSharedMem=NULL;
      return FALSE;
    }
    sprintf(filename," -C%d,%d",lValue1,lValue2);
    strncat(text,filename,strlen(filename));

    /* Check if we have a source drive for previous image */
    strncat(text," -M",4);
    //if(ulFlags & IDCDT_USEWRITER) {
    if(1) {
      /* We use the writer as source. Check if 'dev=x,y,z' is given as cdrecord options */

      /*      chrPtr=strstr(chrCDROptions,"dev=");
              if(NULL==chrPtr) {
              sprintf(text,"There is no device given as a CD-writer . Go to the 'CD-type selection' page in \
              the settings notebook to chose a drive letter instead or go to the 'CDRecord/2' page to include a 'dev=x,y,z'-\
              statement in the options entryfield.");  
              WinMessageBox(  HWND_DESKTOP, HWND_DESKTOP,
              text,
              "Imagefile creation error",
              0UL, MB_OK | MB_ICONEXCLAMATION |MB_MOVEABLE);
              DosFreeMem(pvSharedMem);
              pvSharedMem=NULL;
              return FALSE;
              }*/
      /* Find end of 'dev=...' statement */
      /*chrPtr2=chrPtr;
        while(*chrPtr2!=' '&& *chrPtr2!=0)
        chrPtr2++;
        filename[0]=*chrPtr2;
        *chrPtr2=0;
        chrPtr=chrPtr2;*/
      /* Find begin of traget,lun */
      //chrPtr-=3;

      /* Add target,lun statement to cmd-line */
      sprintf(chrDev,"%d,%d",iTarget,iLun);
      strncat(text,chrDev,strlen(chrDev));
      *chrPtr2=filename[0];
    }/* if(ulFlags & IDCDT_USEWRITER) */
    else {
      /* We use a drive letter as source drive */
      strncat(text,chosenMultiSessionCD,sizeof(chosenMultiSessionCD));
    }
    /* We have all information. Build the rest of the cmd-line */
  }/* end of if((ulFlags & IDCDT_MULTISESSION) && !(ulFlags & IDCDT_FIRSTSESSION)) */
  else {  }

  if(outputName!=NULL) {
    /* Name of output file given */
    strncat(text," -o",4);
    strncat(text,outputName,strlen(outputName));
  }

  /*****************************/
  /* Query input files/folders */
  /* and build cmd line        */
  /*****************************/
  strncat(text," ",2);
  /* Find last space. It's the beginning of the file names */
  chrPtrParam=strrchr(text,' ');

  ulSize=sizeof(filename);
  /* Query name of this folder */
  if(!thisPtr->wpQueryRealName(filename,&ulSize,TRUE)) {
    sprintf(text,"Error while quering the foldername!");  
    WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"Image file creation",
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;

    return FALSE;/* There's an error */
  }
  /* Add the folder name enclosed in " " so spaces are allowed */
  strncat(text,"\"",sizeof("\""));
  strncat(text,filename,ulSize);
  strncat(text,"\"",sizeof("\""));

  hwnd=WinQueryWindow(thisPtr->hwndMkisofsMain,QW_PARENT); // Get folder HWND
  /* Get hwnd of folder container */ 
  hwnd=WinWindowFromID(hwnd,FID_CLIENT);
  if(!WinIsWindow(WinQueryAnchorBlock(HWND_DESKTOP),hwnd)) {
    sprintf(text,"Error while quering the container HWND!");  
    WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"void CWDataFolder::cwCreateImage()",
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;
    return FALSE;
  }
  mrc=(PMINIRECORDCORE)WinSendMsg(hwnd,CM_QUERYRECORD,NULL,
                                  MPFROM2SHORT(CMA_FIRST,CMA_ITEMORDER));
  if(!mrc){ 
    sprintf(text,"Error! The folder is empty!");  
    WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"Image file creation",
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;
    return FALSE;
  }
  
  /* Enumerate folder contents. We only use filesystem objects. If we find a shadow we follow
     the link and use the linked object if it's a filesystem object */
  while(mrc) {
    /* Get wps-object-ptr. from container item */
    contentObject=(WPObject*)OBJECT_FROM_PREC(mrc);//Get object
    if(contentObject) {
      /* Check if it's a shadow */
      if(somResolveByName(contentObject,"wpQueryShadowedObject")) 
        /* Yes, it's a shadow. Get file system object or NULL */
        contentObject=thisPtr->cwGetFileSystemObject(contentObject);
      else 
        /* It's no shadow, so do nothing. If it's a filesystem object it's included in our list
           because we yet provide this folder as input directory. If it's any other object (abstract)
           mkisofs will skip it because it uses the filesystem for querying directory contents. */
        contentObject=NULL;
    }
    if(contentObject){
      /* It's a file system object */
      strncat(text," \"",sizeof(" \""));
      /* Check if it's folder */
      if(somResolveByName(contentObject,"wpQueryContent")) {
        /* It's a folder. Query the name. */
        ulSize=sizeof(filename);
        ((WPFileSystem*)contentObject)->wpQueryRealName(filename,&ulSize,FALSE);
        /* Add the graft point to the command line. If not done, mkisofs will include the contents
           of the folder in the image, not the folder itself */ 
        if((strlen(text)+strlen(filename)+4)>=SHAREDMEM_SIZE) {
          sprintf(text,"Error! The length of the cmd-line exceeds %d Bytes!!\n",SHAREDMEM_SIZE);  
          WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"Image file creation",
                          0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          DosFreeMem(pvSharedMem);
          pvSharedMem=NULL;
          return FALSE;
        }       
        /* Iclude the name in " " to allow spaces */
        // strncat(text,"\"",sizeof("\""));
        strncat(text,filename,ulSize);
        strncat(text,"/=",2);        
      }
      /* Query the full path */
      ulSize=sizeof(filename);
      ((WPFileSystem*)contentObject)->wpQueryRealName(filename,&ulSize,TRUE);
      /* Add the name to the command line */ 
      if((strlen(text)+strlen(filename)+2)>=SHAREDMEM_SIZE) {
        sprintf(text,"Error! The length of the cmd-line exceeds %d Bytes!!\n",SHAREDMEM_SIZE);  
        WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"Image file creation",
                        0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
        DosFreeMem(pvSharedMem);
        pvSharedMem=NULL;
        return FALSE;
      }
      strncat(text,filename,ulSize);
      strncat(text,"\"",sizeof("\""));
    }/* end of if(contentObject) */           
    /* Get next container item */
    mrc=(PMINIRECORDCORE)WinSendMsg(hwnd,CM_QUERYRECORD,MPFROMP(mrc),
                                    MPFROM2SHORT(CMA_NEXT,CMA_ITEMORDER));
  }// end of while(mrc)
  /* Add additional 0 for DosExecPgm() */
  strncat(text,"\0",sizeof("\0"));
  /* Change backslashs in sourcefiles */
  changeBackslash(chrPtrParam);  
  return TRUE;
}

BOOL checkHelper(PSZ helperPath) 
{
  struct stat statBuf;
  char text[CCHMAXPATH];
  char title[CCHMAXPATH];
  
  /* Check cdrdao path */
  if(stat(helperPath , &statBuf)==-1) {
    /* text: "Can't find the helper program %s. It should be in the installation directory of Audio/Data-CD-Creator."
       title: "Audio/Data-CD-Creator installation problem!"
       */
    getMessage(title, IDSTR_INSTALLNOVIO, sizeof(title), hAudioResource , HWND_DESKTOP);
    sprintf(text,title,helperPath);
    getMessage(title, IDSTR_INSTALLERRORTITLE, sizeof(title), hAudioResource , HWND_DESKTOP);
    
    WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,title,
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
    return FALSE; 
  }
  
  return TRUE;  
}


/* This function launches the wrapper <wrapperExe>         */
/*  with the params given as a PM-session */
/* in PSZ parameter. PSZ folderPath is the path to put the */
/* write.log. HWND hwnd is the window waiting for the      */
/* WM_APPTERMINATE message                                 */
ULONG launchPMWrapper(PSZ parameter, PSZ folderPath, PSZ wrapperExe, PSZ pszTitle="CDRecord/2")
{
  STARTDATA startData={0};
  APIRET rc;
  PID pid;
  ULONG ulSessionID=0;
  char chrLoadError[CCHMAXPATH];
  char startParams[CCHMAXPATH*4];
  char exename[CCHMAXPATH]={0};
  char chrFolderPath[CCHMAXPATH+10];
  
  memset(&startData,0,sizeof(startData));
  startData.Length=sizeof(startData);
  startData.Related=SSF_RELATED_INDEPENDENT;
  startData.FgBg=SSF_FGBG_FORE;
  startData.TraceOpt=SSF_TRACEOPT_NONE;
  startData.PgmTitle=pszTitle;
    
  sprintf(exename,"%s",buildWrapName(wrapperExe));

  if(!checkHelper(exename))
    return -1;

  startData.PgmName=exename;
  startData.InheritOpt=SSF_INHERTOPT_SHELL;
  startData.SessionType=SSF_TYPE_PM;
  startData.PgmControl=0;
  startData.InitXPos=30;
  startData.InitYPos=30;
  startData.InitXSize=500;
  startData.InitYSize=400;
  startData.ObjectBuffer=chrLoadError;
  startData.ObjectBuffLen=(ULONG)sizeof(chrLoadError);

  /* Put the exe-path between " " to make sure, spaces are handled correctly */
  //sprintf(chrCDRexe,"\"%s\"",chrCDRecord);
  strcpy(chrFolderPath,folderPath);
  sprintf(startParams,"\"%s\" \"%s\" %s",
          chrInstallDir,chrFolderPath, parameter);
  startData.PgmInputs=startParams;

  rc=DosStartSession(&startData,&ulSessionID,&pid);   
  return 0;   
}


/* This function launches the wrapper <wrapperExe>         */
/*  with the params given */
/* in PSZ parameter. PSZ folderPath is the path to put the */
/* write.log. HWND hwnd is the window waiting for the      */
/* WM_APPTERMINATE message                                 */
ULONG launchWrapper(PSZ parameter, PSZ folderPath,HWND hwnd, PSZ wrapperExe, PSZ title="CDRecord/2")
{
  return launchWriter(parameter, folderPath,hwnd, wrapperExe, title);
}

/* This function launches the writer with the params given */
/* in PSZ parameter. PSZ folderPath is the path to put the */
/* write.log. HWND hwnd is the window waiting for the      */
/* WM_APPTERMINATE message                                 */
ULONG launchWriter(PSZ parameter, PSZ folderPath,HWND hwnd, PSZ wrapperExe, PSZ pszTitle="CDRecord/2")
{
  STARTDATA startData={0};
  APIRET rc;
  PID pid;
  ULONG ulSessionID=0;
  char chrLoadError[CCHMAXPATH];
  char startParams[CCHMAXPATH*4];
  char exename[CCHMAXPATH]={0};
  char chrCDRexe[CCHMAXPATH+10];
  
  memset(&startData,0,sizeof(startData));
  startData.Length=sizeof(startData);
  startData.Related=SSF_RELATED_INDEPENDENT;
  startData.FgBg=SSF_FGBG_FORE;
  startData.TraceOpt=SSF_TRACEOPT_NONE;
  startData.PgmTitle=pszTitle;
    
  sprintf(exename,"%s",buildWrapName(wrapperExe));

  startData.PgmName=exename;
  startData.InheritOpt=SSF_INHERTOPT_SHELL;
  startData.SessionType=SSF_TYPE_WINDOWABLEVIO;
  startData.PgmControl=0;
  if(lCDROptions&IDCDR_HIDEWINDOW)
    startData.PgmControl|=SSF_CONTROL_INVISIBLE;//|SSF_CONTROL_MAXIMIZE|SSF_CONTROL_NOAUTOCLOSE;
  if(!(lCDROptions&IDCDR_CLOSEWINDOW))
    startData.PgmControl|=SSF_CONTROL_NOAUTOCLOSE;
  startData.InitXPos=30;
  startData.InitYPos=30;
  startData.InitXSize=500;
  startData.InitYSize=400;
  startData.ObjectBuffer=chrLoadError;
  startData.ObjectBuffLen=(ULONG)sizeof(chrLoadError);

  /* Put the exe-path between " " to make sure, spaces are handled correctly */
  sprintf(chrCDRexe,"\"%s\"",chrCDRecord);
  sprintf(startParams,"%d %s %s %s",hwnd,chrCDRexe,folderPath ,parameter);

  startData.PgmInputs=startParams;

  rc=DosStartSession(&startData,&ulSessionID,&pid);   
  return 0;   
}

ULONG launchMsInfo(HWND hwnd)
{
  /* hwnd:    notification window
     msinfo.exe sends a WM_APPTERMINATIONNOTIFY msg to that window which contains in mp1/mp2 the
     two numbers necessary for creating of the second session.
     The correct cmd-line for calling cdrecord/2 is build by msinfo. So we supply only the generic
     cdrecord/2 information.
   */

  STARTDATA startData={0};
  APIRET rc;
  PID pid;
  ULONG ulSessionID=0;
  PSZ pszTitle="Query multisession info";
  char chrLoadError[CCHMAXPATH];
  char startParams[CCHMAXPATH*4];
  char exename[CCHMAXPATH]={0};
    
  memset(&startData,0,sizeof(startData));
  startData.Length=sizeof(startData);
  startData.Related=SSF_RELATED_INDEPENDENT;
  startData.FgBg=SSF_FGBG_BACK;
  startData.TraceOpt=SSF_TRACEOPT_NONE;
  startData.PgmTitle=pszTitle;
    
  sprintf(exename,"%s",buildWrapName("msinfo.exe"));
  startData.PgmName=exename;
  startData.InheritOpt=SSF_INHERTOPT_SHELL;
  startData.SessionType=SSF_TYPE_WINDOWABLEVIO;
  startData.PgmControl=SSF_CONTROL_INVISIBLE;//|SSF_CONTROL_MAXIMIZE|SSF_CONTROL_NOAUTOCLOSE;
  startData.InitXPos=30;
  startData.InitYPos=30;
  startData.InitXSize=500;
  startData.InitYSize=400;
  startData.ObjectBuffer=chrLoadError;
  startData.ObjectBuffLen=(ULONG)sizeof(chrLoadError);
  //  sprintf(startParams,"%d %s %s",hwnd, chrCDRecord, chrCDROptions);
  sprintf(startParams,"%d %s dev=%d,%d,%d %s",hwnd, chrCDRecord, iBus, iTarget, iLun, chrDataCDROptions);
  startData.PgmInputs=startParams;
  rc=DosStartSession(&startData,&ulSessionID,&pid);   
  return 0;   
}

ULONG launchMkisofs(HWND hwnd)
{
  /* hwnd:    notification window
   */

  STARTDATA startData={0};
  APIRET rc;
  PID pid;
  ULONG ulSessionID=0;
  PSZ pszTitle="Create ISO-Image";
  char chrLoadError[CCHMAXPATH];
  char startParams[CCHMAXPATH*4];
  char exename[CCHMAXPATH]={0};
    
  memset(&startData,0,sizeof(startData));
  startData.Length=sizeof(startData);
  startData.Related=SSF_RELATED_INDEPENDENT;
  startData.FgBg=SSF_FGBG_FORE;
  startData.TraceOpt=SSF_TRACEOPT_NONE;
  startData.PgmTitle=pszTitle;
    
  sprintf(exename,"%s",buildWrapName("lnchmkis.exe"));
  startData.PgmName=exename;
  startData.InheritOpt=SSF_INHERTOPT_SHELL;
  startData.SessionType=SSF_TYPE_WINDOWABLEVIO;
  startData.PgmControl=0;
  if(lMKOptions&IDMK_HIDEWINDOW)
    startData.PgmControl|=SSF_CONTROL_INVISIBLE;
  if(!(lMKOptions&IDMK_CLOSEWINDOW))
    startData.PgmControl|=SSF_CONTROL_NOAUTOCLOSE;//|SSF_CONTROL_MAXIMIZE|SSF_CONTROL_NOAUTOCLOSE;
  startData.InitXPos=30;
  startData.InitYPos=30;
  startData.InitXSize=500;
  startData.InitYSize=400;
  startData.ObjectBuffer=chrLoadError;
  startData.ObjectBuffLen=(ULONG)sizeof(chrLoadError);
  sprintf(startParams,"%d",hwnd);
  startData.PgmInputs=startParams;
  rc=DosStartSession(&startData,&ulSessionID,&pid);   
  return rc;   
}

















