/*
 * This file is (C) Chris Wohlgemuth 1999
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define HF_STDOUT 1      /* Standard output handle */
#define HF_STDERR 2
#define PIPESIZE  256
 
#include <process.h>

#include "audiofolder.hh"
#include "audiofolderhelp.h"

#pragma SOMAsDefault(off)
#include <sys\types.h>
#include <sys\stat.h>
#pragma SOMAsDefault(pop)

#include <stdio.h>
#include <string.h>//for memccpy()
#include <stdlib.h>
#include "progbars.h"
extern PVOID pvSharedMem;

extern char chrImage[CCHMAXPATH];/* Path to iso-image */
extern LONG lMKOptions;
extern BOOL setupDone;
extern char lnchParam[1024];
extern char chrInstallDir[CCHMAXPATH];
extern char chrDataCDROptions[CCHMAXPATH];
extern char chrCDRecord[CCHMAXPATH];/* Path to cdrecord */
extern char chrMpg123Path[CCHMAXPATH];

extern int iBus;
extern int iTarget;
extern int iLun;

extern ATOM atomStartWrite;
extern HMODULE hDataResource;
extern HMODULE hAudioResource;

HMODULE queryModuleHandle(void);
ULONG messageBox( char* text, ULONG ulTextID , LONG lSizeText,
                  char* title, ULONG ulTitleID, LONG lSizeTitle,
                  HMODULE hResource, HWND hwnd, ULONG ulFlags);
void getMessage(char* text,ULONG ulID, LONG lSizeText, HMODULE hResource,HWND hwnd);
PSZ buildWrapName(PSZ wrapperName);
ULONG launchWrapper(PSZ parameter, PSZ folderPath,HWND hwnd, PSZ wrapperExe, PSZ pszTitle);
ULONG launchMkisofs(HWND hwnd);


static HWND hwndOnTheFly=0;

/* This Proc handles  */
MRESULT EXPENTRY mkisofsObjectProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  CWDataFolder * thisPtr;
  char text[CCHMAXPATH];
  char title[CCHMAXPATH];

  char *textPtr;
  char *textPtr2; 

  switch (msg)
    {
    case WM_APPTERMINATENOTIFY:
      thisPtr=(CWDataFolder*)WinQueryWindowULong(hwnd,QWL_USER);/* Get object ptr */
      if(thisPtr) {
        thisPtr->cwSetStatusText("");/* Clear staus line */      
        //if(thisPtr->pvSharedMem) {/* We just created an image or wrote on the fly */
        switch(LONGFROMMP(mp1)) {
        case ACKEY_IMAGEONLY:
          if(!(LONGFROMMP(mp2)))
            WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, "CD-ROM image file created. You may burn this file on a CDR now.",
                           "Create image file.",
                           0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          else
            WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, "Error while creating the image file.",
                           "Create image file.",
                           0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          break;
        case ACKEY_WRITEONLY:
          if(!(LONGFROMMP(mp2)))
            WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, "Image file written by CDRecord/2.",
                           "Write image file.",
                           0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          else
            WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, "Error while writing the image file. Examine 'write.log.'",
                           "Write image file.",
                           0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          break;
        case ACKEY_PRINTSIZE:
          if(LONGFROMMP(mp2)==0) {
            /* Text: "There was an error while querying the image size. If merging a previous session the
               CD with the session must be in the writer."
               Title:  "Check size"
               */
            messageBox( text, IDSTRD_CHECKSIZEERRORMULTI , sizeof(text),
                        title, IDSTRD_CHECKSIZETITLE, sizeof(title),
                        hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
          }
          else {
            sprintf(text,"Estimated imagesize is %d.%0.3d.%0.3d bytes",
                    LONGFROMMP(mp2)*2048/1000000,(LONGFROMMP(mp2)*2048%1000000)/1000,LONGFROMMP(mp2)*2048%1000);
            thisPtr->cwSetStatusText(text);
            DosBeep(2000,200);
          }
          break;
        case ACKEY_CDSIZE:
          sprintf(text,"Free CD space is %d.%0.3d.%0.3d bytes",
                  LONGFROMMP(mp2)*2048/1000000,(LONGFROMMP(mp2)*2048%1000000)/1000,LONGFROMMP(mp2)*2048%1000);
          thisPtr->cwSetStatusText(text);
          DosBeep(2000,200);
          //WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, "Print size done",
          //             "Print size",
          //             0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
          break;

        default:
          break;
        }/* switch */

      }/* if(thisPtr) */           
      WinPostMsg(hwnd,WM_QUIT,0,0);
      break;
    default:
      if(msg == atomStartWrite) {
        thisPtr=(CWDataFolder*)PVOIDFROMMP(mp1);
        WinSetWindowULong(hwnd,QWL_USER,(ULONG)thisPtr);/* Save object ptr */
        if(!thisPtr) {
          /* Error: quit */
          WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
          return 0;
        }
        switch(LONGFROMMP(mp2)) {   
        case ACKEY_IMAGEONLY:
          /* Start mkisofs */
          if(launchMkisofs(hwnd))
            WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
          return FALSE;
        case ACKEY_PRINTSIZE:
          /* calculating image size */
          launchWrapper("", "", hwnd,"prntsize.exe","Query image size");
          return FALSE;
        case ACKEY_CDSIZE:
          /* querying cd size */
          /*    sprintf(text,"%s",chrCDROptions);
                textPtr=strstr(strlwr(text),"dev=");
                if(!textPtr) {
                WinPostMsg(hwnd,WM_QUIT,0,0);
                return FALSE;
                }
                textPtr2=strchr(textPtr,' ');
                if(textPtr2)
                *textPtr2='\0';*/
          sprintf(text,"dev=%d,%d,%d",iBus,iTarget,iLun);
          launchWrapper(text, "", hwnd,"cdsize.exe","Query CD size");
          return FALSE;
        default:
          break;
        }/* switch */
      }/* if(msg == atomStartWrite) */
      return WinDefWindowProc( hwnd, msg, mp1, mp2);
    }
  return FALSE;
}


/* This thread waits for the mkisofs process to end */
void _Optlink mkisofsThreadFunc (void *arg)
{
  HWND hwndMkisofs;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;
  CWDataFolder * thisPtr;

  thisPtr=(CWDataFolder*)arg;
    
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {
      hwndMkisofs=WinCreateWindow(HWND_OBJECT,WC_STATIC,"myMkisofsObj",0,0,0,0,0,NULL,HWND_BOTTOM,12341,NULL,NULL);
      if(hwndMkisofs) {
        WinSubclassWindow(hwndMkisofs,&mkisofsObjectProc);
        /* Window created. */             
        WinPostMsg(hwndMkisofs,atomStartWrite,MPFROMP(thisPtr),MPFROMLONG(ACKEY_IMAGEONLY));

        thisPtr->cwSetStatusText(NULL);

        thisPtr->cwEnableWriteControls(FALSE);
        while(WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))
          WinDispatchMsg(hab,&qmsg);
        WinDestroyWindow(hwndMkisofs);
      }

      WinDestroyMsgQueue(hmq);
    }
    WinTerminate(hab);
  }
  /* Free cmd-line mem */
  if(pvSharedMem) {
    DosFreeMem(pvSharedMem);
    pvSharedMem=0;
  }
  thisPtr->cwEnableWriteControls(TRUE);
  sprintf(thisPtr->chrStatusText,"");
  thisPtr->cwEnableWriteControls(TRUE);
}


/* This thread waits for the image size process to end */
void _Optlink printSizeThreadFunc (void *arg)
{
  HWND hwndMkisofs;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;
  char text[CCHMAXPATH+100];

  CWDataFolder * thisPtr;

  thisPtr=(CWDataFolder*)arg;
    
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {
      hwndMkisofs=WinCreateWindow(HWND_OBJECT,WC_STATIC,"myPrintSizeObj",0,0,0,0,0,NULL,HWND_BOTTOM,12343,NULL,NULL);
      if(hwndMkisofs) {
        WinSubclassWindow(hwndMkisofs,&mkisofsObjectProc);
        /* Window created. */             
        WinPostMsg(hwndMkisofs,atomStartWrite,MPFROMP(thisPtr),MPFROMLONG(ACKEY_PRINTSIZE));
        thisPtr->cwSetStatusText(NULL);
        thisPtr->cwEnableWriteControls(FALSE);
        while(WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))
          WinDispatchMsg(hab,&qmsg);
        WinDestroyWindow(hwndMkisofs);
      }
      WinDestroyMsgQueue(hmq);
    }
    WinTerminate(hab);
  }
  thisPtr->cwEnableWriteControls(TRUE);
  sprintf(thisPtr->chrStatusText,"");
  /* Free cmd-line mem */
  if(pvSharedMem) {
    DosFreeMem(pvSharedMem);
    pvSharedMem=0;
  }
  /*
  if(thisPtr->pvSharedMem) {
    DosFreeMem(thisPtr->pvSharedMem);
    thisPtr->pvSharedMem=0;
  }*/
  //  DosBeep(100,2000);
}


void _Optlink cdSizeThreadFunc(void *arg)
{
  HWND hwndMkisofs;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;
  char text[CCHMAXPATH+100];
  CWDataFolder * thisPtr;

  thisPtr=(CWDataFolder*)arg;
    
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {
      hwndMkisofs=WinCreateWindow(HWND_OBJECT,WC_STATIC,"cdSizeObj",0,0,0,0,0,NULL,HWND_BOTTOM,12343,NULL,NULL);
      if(hwndMkisofs) {
        WinSubclassWindow(hwndMkisofs,&mkisofsObjectProc);
        /* Window created. */             
        WinPostMsg(hwndMkisofs,atomStartWrite,MPFROMP(thisPtr),MPFROMLONG(ACKEY_CDSIZE));
        thisPtr->cwEnableWriteControls(FALSE);
        while(WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))
          WinDispatchMsg(hab,&qmsg);
        WinDestroyWindow(hwndMkisofs);
      }
      WinDestroyMsgQueue(hmq);
    }
    WinTerminate(hab);
  }
  thisPtr->cwEnableWriteControls(TRUE);
  sprintf(thisPtr->chrStatusText,"");
}

/* This Proc handles the calculation of the playtime of waves and mp3s */
MRESULT EXPENTRY playTimeObjectProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
  char text[CCHMAXPATH*2];
  char title[CCHMAXPATH];
  char *textPtr;
  char *textPtr2; 

  class playTimeInfo {
  public:
    CWAudioFolder *thisPtr;
    HWND  hwndCnr;
    PMINIRECORDCORE mrc;   
  };

  playTimeInfo * ptiInfo;
  HWND hwndFrame;
  WPObject * contentObject;
  char name[CCHMAXPATH];
  ULONG ulNameSize;
  ULONG ulSize;
  ULONG ulFlags;

  switch (msg)
    {
    case WM_APPTERMINATENOTIFY:
      ptiInfo=(playTimeInfo*)WinQueryWindowULong(hwnd,QWL_USER);/* Get object ptr */
      if(!ptiInfo || !ptiInfo->thisPtr) {
        WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
        break;
      }
      ptiInfo->thisPtr->ulTrackSize+=LONGFROMMP(mp2);
      switch(LONGFROMMP(mp1)) {
      case ACKEY_PLAYTIME:
        if(ptiInfo->mrc  && (int)ptiInfo->mrc!=-1){ 
          /* Get next container item */
          ptiInfo->mrc=(PMINIRECORDCORE)WinSendMsg(ptiInfo->hwndCnr,CM_QUERYRECORD,MPFROMP(ptiInfo->mrc),
                                                   MPFROM2SHORT(CMA_NEXT,CMA_ITEMORDER));
          if(ptiInfo->mrc && (int)ptiInfo->mrc!=-1){ 
            /* Get wps-object-ptr. from container item */
            contentObject=(WPObject*)OBJECT_FROM_PREC(ptiInfo->mrc);//Get object
            ulNameSize=sizeof(name);
            if(contentObject) {
              /* Get file system object or NULL */
              contentObject=ptiInfo->thisPtr->cwGetFileSystemObject(contentObject);
            }
            if(contentObject){
              /* It's a file system object */
              /* Check, if it's a mp3 file */
              if(ptiInfo->thisPtr->cwIsMp3File(contentObject)){
                /* Yes, query the full path */
                ((WPFileSystem*)contentObject)->wpQueryRealName(name,&ulNameSize,TRUE);
                /* Launch helper */
                sprintf(text,"\"%s\" \"%s\"",chrMpg123Path,name);
                launchWrapper(text, "", hwnd,"mp3size.exe","Query mp3 size"); 
                return FALSE;
              } /* End of cwIsMp3File(contentObject) */           
            }/* end of if(contentObject) */           
          }// end of if(mrc)
        }// end of if(mrc)
        if( ptiInfo->mrc && (int)ptiInfo->mrc!=-1) {
          /* We have more container items so continue */
          WinPostMsg(hwnd,WM_APPTERMINATENOTIFY,MPFROMLONG(ACKEY_PLAYTIME),MPFROMLONG(0));
          return FALSE;
        }          
        ptiInfo->thisPtr->cwSetStatusTime(ptiInfo->thisPtr->ulTrackSize);
        delete(ptiInfo);
        WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
        break;
      default:
        break;
      }/* switch */
      break;
    default:
      if(msg == atomStartWrite) {
        ptiInfo=new(playTimeInfo);        
        if(!ptiInfo) {
          /* Error: quit */
          WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
          return 0;
        }
        ptiInfo->thisPtr=(CWAudioFolder*)PVOIDFROMMP(mp1);
        WinSetWindowULong(hwnd,QWL_USER,(ULONG)ptiInfo);/* Save object ptr */
        if(!ptiInfo->thisPtr) {
          /* Error: quit */
          WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
          return 0;
        }
        switch(LONGFROMMP(mp2)) {   
        case ACKEY_PLAYTIME:
          /* Name: "Calculating Play time. Please wait..." */
          getMessage(name, IDSTRA_CALCPLAYTIME, sizeof(name),  hAudioResource, ptiInfo->thisPtr->hwndStatusFrameCtl);
          
          ptiInfo->thisPtr->cwSetStatusText(name);
          /* First collect the wave files */
          ulSize=0;
          hwndFrame=WinQueryWindow(ptiInfo->thisPtr->hwndStatusFrameCtl,QW_PARENT);
          /* Get hwnd of folder container */ 
          ptiInfo->hwndCnr=WinWindowFromID(hwndFrame,FID_CLIENT);
          if(ptiInfo->hwndCnr){ /* Catch error */
            /* Get first container item of our folder */
            ptiInfo->mrc=(PMINIRECORDCORE)WinSendMsg(ptiInfo->hwndCnr,CM_QUERYRECORD,NULL,
                                                     MPFROM2SHORT(CMA_FIRST,CMA_ITEMORDER));
            if(ptiInfo->mrc && (int)ptiInfo->mrc!=-1){ 
              while(ptiInfo->mrc) {
                /* Get wps-object-ptr. from container item */
                contentObject=(WPObject*)OBJECT_FROM_PREC(ptiInfo->mrc);//Get object
                ulNameSize=sizeof(name);
                if(contentObject) {
                  /* Get file system object or NULL */
                  contentObject=ptiInfo->thisPtr->cwGetFileSystemObject(contentObject);
                }
                if(contentObject){
                  /* It's a file system object */
                  /* Check, if it's a wave file */
                  if(ptiInfo->thisPtr->cwIsWaveFile(contentObject)){
                    /* Yes, query the full path */
                    ((WPFileSystem*)contentObject)->wpQueryRealName(name,&ulNameSize,TRUE);
                    /* Add tracksize of wavefile to totalsize */ 
                    ulSize+=((WPFileSystem*)contentObject)->wpQueryFileSize()-44;
                  }           
                }/* end of if(contentObject) */           
                /* Get next container item */
                ptiInfo->mrc=(PMINIRECORDCORE)WinSendMsg(ptiInfo->hwndCnr,CM_QUERYRECORD,MPFROMP(ptiInfo->mrc),
                                                         MPFROM2SHORT(CMA_NEXT,CMA_ITEMORDER));
              }// end of while(mrc)
            }// end of if(mrc)
          } // end of if(ptiInfo->hwndCnr)
          /* End of query wave files */
          
          ptiInfo->thisPtr->ulTrackSize=ulSize;
          /* Add playtime of mp3files to totalsize */
          ulFlags=ptiInfo->thisPtr->cwQueryWriteFlags();
          if(strlen(chrMpg123Path)&&!(ulFlags&IDWF_DAO)) {// Only if mpg123 avaiable and DAO not selected */
            if(ptiInfo->hwndCnr){ /* Catch error */
              /* Get first container item of our folder */
              ptiInfo->mrc=(PMINIRECORDCORE)WinSendMsg(ptiInfo->hwndCnr,CM_QUERYRECORD,NULL, MPFROM2SHORT(CMA_FIRST,CMA_ITEMORDER));
              if(ptiInfo->mrc){ 
                while(ptiInfo->mrc) {
                  /* Get wps-object-ptr. from container item */
                  contentObject=(WPObject*)OBJECT_FROM_PREC(ptiInfo->mrc);//Get object
                  ulNameSize=sizeof(name);
                  if(contentObject) {
                    /* Get file system object or NULL */
                    contentObject=ptiInfo->thisPtr->cwGetFileSystemObject(contentObject);
                  }
                  if(contentObject){
                    /* It's a file system object */
                    /* Check, if it's a mp3 file */
                    if(ptiInfo->thisPtr->cwIsMp3File(contentObject)){
                      /* Yes, query the full path */
                      ((WPFileSystem*)contentObject)->wpQueryRealName(name,&ulNameSize,TRUE);
                      /* Launch helper */
                      sprintf(text,"\"%s\" \"%s\"",chrMpg123Path,name);
                      launchWrapper(text, "", hwnd,"mp3size.exe","Query mp3 size"); 
                      return FALSE;
                    } /* End of cwIsMp3File(contentObject) */           
                  }/* end of if(contentObject) */           
                  /* Get next container item */
                  ptiInfo->mrc=(PMINIRECORDCORE)WinSendMsg(ptiInfo->hwndCnr,CM_QUERYRECORD,MPFROMP(ptiInfo->mrc),
                                                           MPFROM2SHORT(CMA_NEXT,CMA_ITEMORDER));
                }// end of while(mrc)
              }// end of if(mrc)
            } 
          }// end of if(strlen(chrMpg123Path))
          ptiInfo->thisPtr->cwSetStatusTime(ptiInfo->thisPtr->ulTrackSize);
          delete(ptiInfo);
          WinPostMsg(hwnd,WM_QUIT,(MPARAM)0,(MPARAM)0);
          return FALSE;
        default:
          break;
        }/* switch */
      }/* if(msg == atomStartWrite) */
      return WinDefWindowProc( hwnd, msg, mp1, mp2);
    }
  return FALSE;
}


/* This thread calculates the playtime of all wave and mp3 files */
void _Optlink playTimeThreadFunc(void *arg)
{
  HWND hwndPlayTime;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;
  char text[CCHMAXPATH+100];
  CWAudioFolder * thisPtr;

  thisPtr=(CWAudioFolder*)arg;
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {
      hwndPlayTime=WinCreateWindow(HWND_OBJECT,WC_STATIC,"playTimeObj",0,0,0,0,0,NULL,HWND_BOTTOM,12343,NULL,NULL);
      if(hwndPlayTime) {
        WinSubclassWindow(hwndPlayTime,&playTimeObjectProc);
        /* Window created. */             
        WinPostMsg(hwndPlayTime,atomStartWrite,MPFROMP(thisPtr),MPFROMLONG(ACKEY_PLAYTIME));
        while(WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))
          WinDispatchMsg(hab,&qmsg);
        WinDestroyWindow(hwndPlayTime);
      }
      WinDestroyMsgQueue(hmq);
    }
    WinTerminate(hab);
  }
}


/* This thread handles the -scanbus stuff                */ 
/* It is created when the user presses the 'scan' button */
/* on the general page in the settings notebook          */
void _Optlink scanbusThreadFunc(void *arg)
{
  HWND hwndScanBus;
  HAB  hab;
  HMQ  hmq;
  QMSG qmsg;

  char exeName[CCHMAXPATH+20];
  char exeError[CCHMAXPATH];
  int rc;
  HFILE hfNew;
  HFILE hfSave;
  HPIPE hpR, hpW;
  RESULTCODES ChildRC= {0};
  PID pidChild;
  ULONG cbRead, cbWritten;
  char *text;
  char chrBuf;

  ULONG count;
  int bus,target,lun;
  char vendor[CCHMAXPATH];
  char type[CCHMAXPATH];
  BOOL bBus=FALSE;
  BOOL bFirst=TRUE;

  hwndScanBus=(HWND)arg;
  
  hab=WinInitialize(0);
  if(hab) {
    hmq=WinCreateMsgQueue(hab,0);
    if(hmq) {

      WinPostMsg(WinWindowFromID(hwndScanBus,IDDD_DEVICESELECTION),LM_DELETEALL,MPFROMLONG(0),MPFROMLONG(0));

      strcpy(exeName,chrCDRecord);
      text = strchr(exeName,'\0')+1;
      strcpy(text,"-scanbus");
      *(strchr(text,'\0')+1) = '\0';

      /* Redirect stdout */ 
      hfSave = -1;// Get new handle
      hfNew = HF_STDOUT;
  
      if(!DosDupHandle(HF_STDOUT, &hfSave)){    /* Saves standard output handle */
        if(!DosCreatePipe(&hpR, &hpW, PIPESIZE)){ /* Creates pipe            */      
          if(!DosDupHandle(hpW, &hfNew)){           /* Duplicates standard output handle */
            /* start CDRecord/2 */
            rc=DosExecPgm(exeError,sizeof(exeError),EXEC_ASYNCRESULT,exeName,NULL,&ChildRC,exeName);
            sprintf(exeError,"DosExecPgm returned: %d\n",rc);
            // WinPostMsg(WinWindowFromID(hwndScanBus,IDDD_DEVICESELECTION),LM_INSERTITEM,MPFROMSHORT(LIT_END),(MPARAM)exeError);
            DosClose(hpW);                       /* Closes write handle to ensure     */
          
            DosDupHandle(hfSave, &hfNew);        /* Brings stdout back                */
          
            /*
             * Read from the pipe and write to the screen
             * as long as there are bytes to read.
             */
            text=exeName;          
            do {
              //DosRead(hpR, exeName, sizeof(exeName), &cbRead);
              DosRead(hpR, &chrBuf, sizeof(chrBuf), &cbRead);
              if(chrBuf!='\n' && chrBuf!='\r') {
                *text=chrBuf;
                text++;
              }
              else {
                *text='\0';
                text=exeName;          
                if(!strchr(exeName,'*')) {
                  if(bBus && !strstr(exeName,"scsibus")) {
                    count=sscanf(exeName," %d,%d,%d %*[^']'%[^']' '%[^']",&bus,&target,&lun,&vendor,&type);
                    if(count==5) {
                      sprintf(exeError,"%d,%d,%d : %s %s",bus,target,lun,vendor,type);
                      WinPostMsg(WinWindowFromID(hwndScanBus,IDDD_DEVICESELECTION),LM_INSERTITEM,MPFROMSHORT(LIT_END),
                                 (MPARAM)exeError);
                      if(bFirst) {
                        bFirst=FALSE;
                        WinSetWindowText(WinWindowFromID(hwndScanBus,IDDD_DEVICESELECTION),exeError);
                      }
                    }            
                  }
                  if(strstr(exeName,"scsibus"))
                    bBus=TRUE;
                }
              }
            } while(cbRead);   
            DosClose(hpR);
          }
        }
      }/* if(!DosDupHandle(HF_STDERR, &hfSave)) */      
      DosWaitChild(0,DCWW_WAIT,&ChildRC,&pidChild,0);  
    }
    WinTerminate(hab);
  }
}




