/*
 * This file is (C) Chris Wohlgemuth 1999
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "audiofolder.hh"
#include "audiofolderhelp.h"

#include <stdlib.h>
#include <stdio.h>


PVOID pvSharedMem;

char chrImage[CCHMAXPATH]={0};/* Path to iso-image */
char lnchParam[1024];
extern char chrDataCDROptions[CCHMAXPATH];
extern char chrCDRecord[CCHMAXPATH];/* Path to cdrecord */
extern char chrMkisofs[CCHMAXPATH];/* Path to mkisofs */
extern char chrMkisofsOptions[CCHMAXPATH];
extern BOOL bMultiSessionDone;
extern char chosenMultiSessionCD[3];
HMODULE hDataResource;

HMODULE queryModuleHandle(void);
/* Get strings for messagebox from resource-DLL */
ULONG messageBox( char* text, ULONG ulTextID , LONG lSizeText,
                  char* title, ULONG ulTitleID, LONG lSizeTitle,
                  HMODULE hResource, HWND hwnd, ULONG ulFlags);
void getMessage(char* text,ULONG ulID, LONG lSizeText, HMODULE hResource,HWND hwnd);
MRESULT EXPENTRY cdTypeOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY authorOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY cdrecordOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY mkisofsOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY fileNameOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY dataGeneralOptionDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2);
void _Optlink imageWriteThreadFunc (void *arg);
void _Optlink mkisofsThreadFunc (void *arg);
void _Optlink printSizeThreadFunc (void *arg);
void buildDataWriteParam(CWDataFolder* thisPtr, char * text);
BOOL buildMkisofsParam(CWDataFolder* thisPtr, char * text, char * outputName);
PSZ buildWrapName(PSZ wrapperName);
ULONG launchPMWrapper(PSZ parameter, PSZ folderPath, PSZ wrapperExe, PSZ pszTitle);


/********************************************************/
/* New class function which inserts the Cdrecord option */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::cwAddGeneralOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
    char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = dataGeneralOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hDataResource;
  //The ID of the dialog template
  pageinfo.dlgid = IDDLG_GENERAL;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_GENERAL;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "General" */
  getMessage(pageName, IDSTRSETTING_GENERAL, sizeof(pageName),  hDataResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the filename option */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::AddFileNameOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = fileNameOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hDataResource;
  pageinfo.dlgid = IDDLG_FILENAMEOPTIONS;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_FILENAMEOPTIONS;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "Filename options" */
  getMessage(pageName, IDSTRSETTING_FILENAME,sizeof(pageName),  hDataResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the mkisofs setup */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::AddMkisofsOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_STATUSTEXTON;
  //pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = mkisofsOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hDataResource;
  pageinfo.dlgid = IDDLG_MKISOFSSETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_MKISOFSSETUP;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName:  "Mkisofs setup"*/
  //getMessage(pageName, IDSTRSETTING_MKISOFSSETTINGS,sizeof(pageName),  hDataResource, hwndNotebook);
  //pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the Cdrecord option */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::AddCdrecordOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = cdrecordOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hDataResource;
  pageinfo.dlgid = IDDLG_CDRECORDSETUP;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_CDRECORDSETUP;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "CDRecord/2 options" */
  //getMessage(pageName, IDSTRSETTING_CDRSETTINGS,sizeof(pageName),  hDataResource, hwndNotebook);
  /* pageName: "Helper applications" */
  getMessage(pageName, IDSTRSETTING_HELPERAPPLICATIONS, sizeof(pageName),  hDataResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the author          */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::AddAuthorOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = authorOptionDlgProc;
  //The resource DLL
  pageinfo.resid = hDataResource;
  pageinfo.dlgid = IDDLG_AUTHOR;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_AUTHOR;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "CD name and author"*/
  getMessage(pageName, IDSTRSETTING_CDDESCRIPTION,sizeof(pageName),  hDataResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

/********************************************************/
/* New class function which inserts the CD-Type         */
/* page into the settings notebook                      */
/********************************************************/
BOOL CWDataFolder::AddCDTypeOptionPage(HWND hwndNotebook)
{
  PAGEINFO pageinfo;
  char pageName[100];

  //Clear the pageinfo structure
  memset((PCH)&pageinfo, 0, sizeof(PAGEINFO));
  //Fill the pageinfo structure
  pageinfo.cb = sizeof(PAGEINFO);
  pageinfo.hwndPage = NULLHANDLE;
  pageinfo.usPageStyleFlags = BKA_MAJOR | BKA_STATUSTEXTON;
  pageinfo.usPageInsertFlags = BKA_FIRST;
  //We want page numbers
  pageinfo.usSettingsFlags = SETTINGS_PAGE_NUMBERS;
  //The dialog procedure for this page
  pageinfo.pfnwp = cdTypeOptionDlgProc;
 //The resource DLL
  pageinfo.resid = hDataResource;

  pageinfo.dlgid = IDDLG_CDTYPE;
  //We need a pointer to our WPS-object in the dialog procedure
  //to call class functions
  pageinfo.pCreateParams = this;
  //The ID of the help panel for this page
  pageinfo.idDefaultHelpPanel = IDDLG_CDTYPE;
  //Tell the WPS the help library name
  pageinfo.pszHelpLibraryName = AFHELPLIBRARY;
  //We have a major tab so we need a name
  /* pageName: "CD-Type selection" */
  getMessage(pageName, IDSTRSETTING_CDTYPE,sizeof(pageName),  hDataResource, hwndNotebook);
  pageinfo.pszName = pageName;
  
  //Insert the page into the settings notebook
  return wpInsertSettingsPage(hwndNotebook,&pageinfo);
}

void CWDataFolder::cwSetStatusText(PSZ pszText)
{
  if(pszText)
    WinSetWindowText(WinWindowFromID(hwndStatusCntrl,IDST_STATUSTOTALTIME),pszText);
  else
    WinSetWindowText(WinWindowFromID(hwndStatusCntrl,IDST_STATUSTOTALTIME),chrStatusText);
}

ULONG CWDataFolder::cwQueryMkisofsFlags()
{
  return ulMkisofsFlags;
}

ULONG CWDataFolder::cwSetMkisofsFlags(ULONG ulNewMkisofsFlags,ULONG ulMask)
{
  ULONG rc;

  rc=cwQueryMkisofsFlags();
  ulMkisofsFlags=((ulNewMkisofsFlags&ulMask) | (~ulMask & ulMkisofsFlags));

  return rc;/* return previous flags */
}

LONG CWDataFolder::cwQueryPreviousStartSector()
{
  return lPreviousStart;
}

LONG CWDataFolder::cwQueryNextStartSector()
{
  return lNextStart;
}

void CWDataFolder::cwSetPreviousStartSector(LONG lSector)
{
  lPreviousStart=lSector;
}

void CWDataFolder::cwSetNextStartSector(LONG lSector)
{
  lNextStart=lSector;
}

ULONG CWDataFolder::cwQueryCDTypeFlags()
{
  return ulCDTypeFlags;
}

ULONG CWDataFolder::cwSetCDTypeFlags(ULONG ulNewCDTypeFlags,ULONG ulMask)
{
  ULONG rc;

  rc=cwQueryCDTypeFlags();
  ulCDTypeFlags=((ulNewCDTypeFlags&ulMask) | (~ulMask & ulCDTypeFlags));

  return rc;/* return previous flags */
}

void CWDataFolder::cwEnableMultiSessionCntrls(HWND hwnd, BOOL bEnable)
{
  /* hwnd:    Dialog-HWND
     nEnable: New enable state */

  //WinEnableWindow(WinWindowFromID(hwnd,IDCB_FIRSTSESSION),bEnable);
  // WinEnableWindow(WinWindowFromID(hwnd,IDCB_WRITERSOURCE),bEnable);
  //  WinEnableWindow(WinWindowFromID(hwnd,IDDD_SOURCEDRIVE),bEnable);
  WinEnableWindow(WinWindowFromID(hwnd,IDST_PREVIOUSINFO),bEnable);
  //WinEnableWindow(WinWindowFromID(hwnd,IDST_SOURCEDRIVE),bEnable);
  WinEnableWindow(WinWindowFromID(hwnd,IDGB_MERGESESSION),bEnable);//Groupbox
  WinEnableWindow(WinWindowFromID(hwnd,IDST_MERGETEXT),bEnable);//Text
  WinEnableWindow(WinWindowFromID(hwnd,IDSB_PREVIOUSSTART),bEnable);
  WinEnableWindow(WinWindowFromID(hwnd,IDSB_NEXTSTART),bEnable);
  WinEnableWindow(WinWindowFromID(hwnd,IDPB_GETMSINFO),bEnable);

  /*  bEnable?bEnable=FALSE:bEnable=TRUE;
      WinEnableWindow(WinWindowFromID(hwnd,IDRB_TRACKDATA),bEnable);
      WinEnableWindow(WinWindowFromID(hwnd,IDRB_TRACKMODE2),bEnable);
      WinEnableWindow(WinWindowFromID(hwnd,IDRB_TRACKXA1),bEnable);
      WinEnableWindow(WinWindowFromID(hwnd,IDRB_TRACKXA2),bEnable);
      WinEnableWindow(WinWindowFromID(hwnd,IDRB_TRACKCDI),bEnable);*/
}

ULONG CWDataFolder::cwQueryCreateFlags()
{
  return ulCreateFlags;
}

ULONG CWDataFolder::cwSetCreateFlags(ULONG ulNewCreateFlags,ULONG ulMask)
{
  ULONG rc;

  rc=cwQueryCreateFlags();
  ulCreateFlags=((ulNewCreateFlags&ulMask) | (~ulMask & ulCreateFlags));

  return rc;/* return previous flags */
}

void CWDataFolder::cwEnableWriteControls(BOOL bEnable)
{
  RECTL rectl;

  if(!hwndMkisofsMain)
    return;
  WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDRB_WRITEIMAGE) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDRB_ONTHEFLY) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDRB_IMAGEONLY) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDPB_CHECKSIZE) ,bEnable);
  WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDCB_DUMMY) ,bEnable);

  //  if(WinQueryButtonCheckstate(hwndMkisofsMain,IDRB_IMAGEONLY)&1)
  if(WinQueryButtonCheckstate(hwndMkisofsMain,IDRB_IMAGEONLY))
    WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDPB_CREATE) ,bEnable);
  else 
    WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDPB_WRITECD) ,bEnable);

  if(bEnable) {
    WinEnableWindow(WinWindowFromID(hwndMkisofsMain,IDPB_CREATE) ,bEnable);
    WinQueryWindowRect(hwndMkisofsMain,&rectl);
    WinInvalidateRect(hwndMkisofsMain,&rectl,TRUE);
  }
}

void CWDataFolder::cwShowImageName(BOOL bShow)
{
  WinShowWindow(WinWindowFromID(hwndImageName,IDEF_IMAGENAME) ,bShow);
  WinShowWindow(WinWindowFromID(hwndImageName,IDPB_IMAGEBROWSE) ,bShow);
  WinShowWindow(WinWindowFromID(hwndImageName,IDST_IMAGENAME) ,bShow);
}

/* This method is called if we write an already created image */
void CWDataFolder::cwWriteImage()
{
  char * text;
  ULONG ulSize;
  char filename[CCHMAXPATH];
  char* tempText;
  /* For msgbox strings */
  char text2[CCHMAXPATH];
  char title[CCHMAXPATH];
  text=lnchParam;
  ULONG  ulFlags;

  /* Get current filename from entryfield */
  WinQueryWindowText( WinWindowFromID(hwndImageName,IDEF_IMAGENAME),sizeof(chrImage),chrImage);      

  /* Check if we have an output filename */
  if(strlen(chrImage)<=3) {
    /* Text: "You have to specify a valid filename including the path."
       Title: "Write image."
       */
    messageBox( text2, IDSTRD_CHECKSIZEERRORFILENAME , sizeof(text2),
                title, IDSTRD_WRITEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }
  if(pvSharedMem) {
    /* Text: "There is an imagefile creation or write process running."
       Title: "Write image."
       */
    messageBox( text2, IDSTRD_ALREADYWRITING , sizeof(text2),
                title, IDSTRD_WRITEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }

  if(DosAllocSharedMem(&pvSharedMem,"\\SHAREMEM\\MKISOFSCMDLINE",SHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
    /* Text: "Can't allocate shared memory. There's probably already a creation process running."
       Title: "Write image."
       */
    messageBox( text2, IDSTR_ALLOCSHAREDMEMERROR , sizeof(text2),
                title, IDSTRD_WRITEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    /*  WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, "Can't allocate shared Mem!","Write image",
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );*/
    return;
  }
  memset(pvSharedMem,0,SHAREDMEM_SIZE);
  text= (char*)pvSharedMem;

  sprintf(text,"%s\"",chrCDRecord);
  tempText=strchr(text,0);
  if(!tempText) {
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;
    /* Text: "Internal error! Can't find end of command. Aborting..."
       Title: "Write image."
       */
    messageBox( text2, IDSTR_CDRSTRINGERROR , sizeof(text2),
                title, IDSTRD_WRITEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    /*    WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, "Internal error! Can't find end of command. Aborting...","Write image",
                    0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );*/
    return;
  }
  
  /* Build cdrecord/2 cmd-line parameters */  
  buildDataWriteParam(this, tempText+1);
  *tempText=' ';
  /* Check if we perform a dummy write */
  if(sDummy==1)
    strcat(text," -dummy");// Dummy write
  else {
    /* Text: "Dummy is unchecked!\n Do you want to perform a real write?" */
    /* Title: "Write image." */
    getMessage(text2, IDSTR_DUMMYUNCHECKED, sizeof(text2), hDataResource,HWND_DESKTOP);
    getMessage(title, IDSTRD_WRITEIMAGETITLE, sizeof(title), hDataResource,HWND_DESKTOP);
    if(WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text2,title,
                       0UL, MB_YESNO | MB_ICONEXCLAMATION|MB_MOVEABLE )==MBID_NO)
      return;
  }

  ulFlags=cwQueryCDTypeFlags();
  if(!(ulFlags & IDCDT_FIXDISK) && !(ulFlags & IDCDT_MULTISESSION))
    strcat(text," -nofix");

  /* Add the imagefile name */
  strcat(text," ");
  strcat(text,chrImage);

  /* Start cdrecord */
  /*WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"void CWDataFolder::cwWriteImage()",
                  0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
  return;*/

  /* Text: "Writing image %s..." */
  getMessage(text2, IDSTRD_WRITINGIMAGE, sizeof(text2), hDataResource,HWND_DESKTOP);
  sprintf(chrStatusText, text2, chrImage);
  
  ulSize=sizeof(filename);
  wpQueryRealName(filename,&ulSize,TRUE);
  
  /* This PM wrapper checks the image- and CD size and controls the status
     window */ 
  launchPMWrapper(chrImage,filename,"pwrteimg.exe",chrStatusText);

}


void changeBackslash(char* text)
{
  char * chrPtr;
  
  chrPtr=strchr(text,'\\');
  
  while(chrPtr!=NULL) {
    *chrPtr='/';
    chrPtr=strchr(text,'\\');
  }
}

BOOL buildWritePMOnTheFlyCmdLine(CWDataFolder* thisPtr)
{
  char *text;
  char *chrPtrParam;
  ULONG ulFlags;
  char text2[CCHMAXPATH];
  char title[CCHMAXPATH];

  if(pvSharedMem) {
    /* Text: "There is an imagefile creation or write process running."
       Title: "Write on the fly"
       */
    messageBox( text2, IDSTRD_ALREADYWRITING , sizeof(text2),
                title, IDSTRD_ONTHEFLYTITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return FALSE;
  }
  
  ulFlags=thisPtr->cwQueryCDTypeFlags();
  if((ulFlags & IDCDT_MULTISESSION) && !(ulFlags & IDCDT_FIRSTSESSION)) {
    /* We are building the second image of a multisession disk */
    /* Text: "With this version of Data-CD-Creator it isn't possible 
       to write further sessions of a multisession CD on the fly."
       Title: "Write on the fly"
       */
    messageBox( text2, IDSTRD_NOMULTIONTHEFLY , sizeof(text2),
                title, IDSTRD_ONTHEFLYTITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return FALSE;
  }

  if(DosAllocSharedMem(&pvSharedMem,"\\SHAREMEM\\MKISOFSCMDLINE",SHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
    /* Text: "Can't allocate shared memory. There's probably already a creation process running."
       Title: "Write on the fly"
       */
    messageBox( text2, IDSTR_ALLOCSHAREDMEMERROR , sizeof(text2),
                title, IDSTRD_ONTHEFLYTITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return FALSE;
  }
  memset(pvSharedMem,0,SHAREDMEM_SIZE);
  text= (char*)pvSharedMem;

  chrPtrParam=text;
  /* Add the mkisofs command */
  buildMkisofsParam(thisPtr, chrPtrParam, "-print-size");

  /* Add cdrecord command */
  strcat(text," | ");
  /* CDRecord path */
  strcat(text,chrCDRecord);
  strcat(text,"  ");
  chrPtrParam=strrchr(text,' ');
  /* Add cdrecord options */
  buildDataWriteParam(thisPtr, chrPtrParam);

  /* Check if we perform a dummy write */
  if(thisPtr->sDummy==1)
    strcat(text," -dummy");// Dummy write

  //if(!(ulFlags & IDCDT_FIXDISK))
  if(!(ulFlags & IDCDT_FIXDISK) && !(ulFlags & IDCDT_MULTISESSION))
    strcat(text," -nofix");

  /* Tell cdrecord to read from stdin */
  strcat(text," -");

#ifdef DEBUG  
  WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text,"void CWDataFolder::cwWriteOnTheFly(): Built the following command",
                  0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );
#endif

  return TRUE;
}

/* This method starts a PM wrapper which handles all the on the fly stuff */
void CWDataFolder::cwWriteOnTheFly()
{
  char *text;
  char *chrPtrParam;
  ULONG ulFlags;
  ULONG ulSize;
  char filename[CCHMAXPATH];
  char text2[CCHMAXPATH];
  char title[CCHMAXPATH];

  /* First check the -dummy switch and ask the user */
  if(sDummy!=1)
    {
      /* Text: "Dummy is unchecked!\n Do you want to perform a real write?" */
      /* Title: "Write on the fly" */
      getMessage(text2, IDSTR_DUMMYUNCHECKED, sizeof(text2), hDataResource,HWND_DESKTOP);
      getMessage(title, IDSTRD_ONTHEFLYTITLE, sizeof(title), hDataResource,HWND_DESKTOP);
      if(WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, text2,title,
                         0UL, MB_YESNO | MB_ICONEXCLAMATION|MB_MOVEABLE )==MBID_NO)
        return;
    }
  /* Text2: "Writing on the fly..." */
  getMessage(text2, IDSTR_ONTHEFLYWRITING, sizeof(text2), hDataResource,HWND_DESKTOP);
    sprintf(chrStatusText,text2);
  if(!buildWritePMOnTheFlyCmdLine(this))
    return;
  ulSize=sizeof(filename);
  wpQueryRealName(filename,&ulSize,TRUE);

  /* This PM wrapper checks the image- and CD size and controls the status
     window */ 
  launchPMWrapper("",filename,"pmthefly.exe",text2);

  return;
}

void CWDataFolder::cwCreateImage()
{
  char *text;
  char text2[CCHMAXPATH];
  char title[CCHMAXPATH];
  
  if(pvSharedMem) {
    /* Text: "There is an imagefile creation or write process running."
       Title: "Image file creation"
       */
    messageBox( text2, IDSTRD_ALREADYWRITING , sizeof(text2),
                title, IDSTRD_CREATEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }
  
  /* Get current filename from entryfield */
  WinQueryWindowText( WinWindowFromID(hwndImageName,IDEF_IMAGENAME),sizeof(chrImage),chrImage);      

  /* Check if we have an output filename */
  if(strlen(chrImage)<=3) {
    /* Text: "There is an imagefile creation or write process running."
       Title: "You have to specify a valid filename including the path."
       */
    messageBox( text2,IDSTRD_CHECKSIZEERRORFILENAME , sizeof(text2),
                title, IDSTRD_CREATEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    /*   WinMessageBox(  HWND_DESKTOP,   HWND_DESKTOP, "You have to specify a valid filename including the path.",
         "Image file creation",
         0UL, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE );*/
    return;
  }
  /* Check if we have a full path (just a quick check) */
  if(chrImage[1]!=':') {
    /* Text: "You have to specify a full path."
       Title: "Image file creation."
       */
    messageBox( text2, IDSTRD_CHECKSIZEERRORFULLPATH, sizeof(text2),
                title, IDSTRD_CREATEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }

  if(DosAllocSharedMem(&pvSharedMem,"\\SHAREMEM\\MKISOFSCMDLINE",SHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
    /* Text: "Can't allocate shared memory. There's probably already a creation process running."
       Title: "Write on the fly"
       */
    messageBox( text2, IDSTR_ALLOCSHAREDMEMERROR , sizeof(text2),
                title, IDSTRD_CREATEIMAGETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }
  memset(pvSharedMem,0,SHAREDMEM_SIZE);
  text= (char*)pvSharedMem;

  if(!buildMkisofsParam(this, text,chrImage)){
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;
    return;
  }

  /* Text: "Creating image %s..." */
  getMessage(text2, IDSTRD_CREATINGIMAGE, sizeof(text2), hDataResource,HWND_DESKTOP);
  sprintf(chrStatusText, text2, chrImage);

  _beginthread(mkisofsThreadFunc,NULL,8192*16,(void*)this); //Fehlerbehandlung fehlt  

  return;
}

void CWDataFolder::cwGetImageSize()
{
  char *text;
  char *chrFound;
  char text2[CCHMAXPATH];
  char title[CCHMAXPATH];

  if(pvSharedMem) {
    /* Text: "There is an imagefile creation or write process running."
       Title: "Check size"
       */
    messageBox( text2, IDSTRD_ALREADYWRITING , sizeof(text2),
                title, IDSTRD_CHECKSIZETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }
  if(DosAllocSharedMem(&pvSharedMem,"\\SHAREMEM\\MKISOFSCMDLINE",SHAREDMEM_SIZE,PAG_READ|PAG_WRITE|PAG_COMMIT)) {
    /* Text: "Can't allocate shared memory. There's probably already a creation process running."
       Title:  "Check size"
       */
    messageBox( text2, IDSTR_ALLOCSHAREDMEMERROR , sizeof(text2),
                title, IDSTRD_CHECKSIZETITLE, sizeof(title),
                hDataResource, HWND_DESKTOP, MB_OK | MB_ICONEXCLAMATION|MB_MOVEABLE);
    return;
  }
  memset(pvSharedMem,0,SHAREDMEM_SIZE);
  text=(char*)pvSharedMem;

  buildMkisofsParam(this, text,"-print-size");
  /* Remove the -o switch */
  chrFound=strstr(text,"-o-print-size");
  if(!chrFound) {
    DosFreeMem(pvSharedMem);
    pvSharedMem=NULL;
    return;
  }
  *chrFound=' ';
  chrFound++;
  *chrFound=' ';

  /* Text: "Calculating image size. Please wait..." */
  getMessage(chrStatusText, IDSTRD_CALCULATINGIMAGESIZE, sizeof(chrStatusText), hDataResource,HWND_DESKTOP);
  //sprintf(chrStatusText,"Calculating image size. Please wait...");

  _beginthread(printSizeThreadFunc,NULL,8192*16,(void*)this); //Fehlerbehandlung fehlt    
}











