/*
 * This file is (C) Chris Wohlgemuth 1999
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#define INCL_DOSERRORS

#include "audiofolder.hh"
#include "audiofolderhelp.h"

#include <stdlib.h>
#include <stdio.h>

extern HMODULE hDataResource;

HMODULE queryModuleHandle(void);

/* Overriden function which returns our class name */
PSZ M_CWDataFolder::wpclsQueryTitle()
{
  return "Data-CD-Creator";
}

ULONG M_CWDataFolder::wpclsQueryStyle()
{
  /* CWProgFolder returns *_NEVERTEMPLATE but we want a template */
  return M_WPFolder::wpclsQueryStyle();
}

BOOL M_CWDataFolder::wpclsQueryDefaultHelp(PULONG HelpPanelId,PSZ HelpLibrary)
{
	if(HelpLibrary)
		strcpy(HelpLibrary,AFHELPLIBRARY);
	if(HelpPanelId)
		*HelpPanelId= IDHLP_DATACDMAIN;
	
	return TRUE;
}

/* Overriden function which returns the custom icon */
ULONG M_CWDataFolder::wpclsQueryIconData(PICONINFO pIconInfo)
{
  if (pIconInfo)   {
    pIconInfo->fFormat = ICON_RESOURCE;
    /* The function queryModuleHandle() returns the module handle
       of the DLL the class resides in. The resource compiler put
       the icon in the DLL after compiling the class */ 
    //pIconInfo->hmod    = queryModuleHandle();
    pIconInfo->hmod    = hDataResource;
    pIconInfo->resid   = ID_ICONDATAFOLDER;
  } /* endif */
  return ( sizeof(ICONINFO) );
}


/* Overriden function which returns the custom icon */
ULONG M_CWDataFolder::wpclsQueryIconDataN(PICONINFO pIconInfo, ULONG ulIconIndex)
{
  if (pIconInfo)   {
    pIconInfo->fFormat = ICON_RESOURCE;
    /* The function queryModuleHandle() returns the module handle
       of the DLL the class resides in. The resource compiler put
       the icon in the DLL after compiling the class */ 
    //pIconInfo->hmod    = queryModuleHandle();
    pIconInfo->hmod    = hDataResource;
    pIconInfo->resid   = ID_ICONDATAFOLDER2;
  } /* endif */
  return ( sizeof(ICONINFO) );
}

void M_CWDataFolder::wpclsInitData()
{
  COUNTRYCODE country= {0};
  COUNTRYINFO countryInfo= {0};
  ULONG ulInfoLen=0;
  APIRET rc;
  char path[CCHMAXPATH];  
  char* found;
  char buf[CCHMAXPATH];

  /* Call parent */
  M_CWProgFolder::wpclsInitData();

  /* Get the country code of our system and load the  
     resource DLL with the right language */
  do {
    rc=DosQueryCtryInfo(sizeof(countryInfo),&country,&countryInfo,&ulInfoLen);
    if(rc!=NO_ERROR)
      break;
    
    rc=DosQueryModuleName(queryModuleHandle(),sizeof(path),path);
    if(rc!=NO_ERROR)
      break;

    found=strrchr(path,'\\');
    if(!found)
      break;

    *found=0;
    sprintf(buf,"\\cdfld%03d.dll",countryInfo.country);
    strcat(path,buf);
    rc=DosLoadModule(buf,sizeof(buf),path, &hDataResource);
    if(rc==NO_ERROR)
      break;

    /* NLS DLL not found. Try to load default */
    found=strrchr(path,'\\');
    if(!found)
      break;
    
    *found=0;
    sprintf(buf,"\\cdfld001.dll");
    strcat(path,buf);
    rc=DosLoadModule(buf,sizeof(buf),path, &hDataResource);

    if(rc!=NO_ERROR)
      hDataResource=NULLHANDLE;
    break;
  }while(TRUE);
}

void M_CWDataFolder::wpclsUnInitData()
{

  M_CWProgFolder::wpclsUnInitData();
  DosFreeModule(hDataResource);

}









