/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "audiofolderres.h"

/* Global flags for cdrecord saved in cdrecord.ini */
#define IDCDR_HIDEWINDOW   0x0001
#define IDCDR_CLOSEWINDOW 0x0002

/* Key for save/restore of write flags */
#define IDKEY_FDRWRITEFLAGS        4000

/* write flags (Audio) */
#define IDWF_PAD     0x0001
#define IDWF_NOFIX   0x0002
#define IDWF_PREEMP  0x0004
#define IDWF_DUMMY  0x0008
#define IDWF_DAO  0x0010

#define WF_ALLFLAGS (IDWF_PAD|IDWF_NOFIX|IDWF_DUMMY|IDWF_PREEMP|IDWF_DAO)

/* Global mkisofs flags saved in cdrecord.ini */
#define IDMK_HIDEWINDOW   0x0001
#define IDMK_CLOSEWINDOW 0x0002


/* Key for save/restore of mkisofs flags */
#define IDKEY_MKISOFSFLAGS        5000

/* mkisofs flags */
#define IDMK_ALLOW32CHARS     0x0001
#define IDMK_LEADINGPERIODS   0x0002
#define IDMK_JOLIET  0x0004
#define IDMK_ROCKRIDGE  0x0008
#define IDMK_TRANSTABLE  0x0010
#define IDMK_LOGFILE  0x0020
#define IDMK_ALLFILES  0x0040
#define MK_ALLFLAGS (IDMK_ALLOW32CHARS|IDMK_LEADINGPERIODS|IDMK_JOLIET|IDMK_ROCKRIDGE|IDMK_TRANSTABLE|IDMK_LOGFILE | IDMK_ALLFILES)

/* Key for save/restore of create flags */
#define IDKEY_CREATEFLAGS        5500

/* Create flags */
#define IDCR_IMAGEONLY 0x0001
#define IDCR_WRITEIMAGE 0x0002
#define IDCR_ONTHEFLY    0x0004
#define IDCR_ASK           0x0008

/* Key for save/restore of cdtype flags */
#define IDKEY_CDTYPEFLAGS        6000

/* CD-type flags */
#define IDCDT_MULTISESSION     0x0001
#define IDCDT_FIRSTSESSION     0x0002
#define IDCDT_USEWRITER        0x0004
#define IDCDT_TRACKDATA        0x0008
#define IDCDT_TRACKMODE2       0x0010
#define IDCDT_TRACKXA1         0x0020
#define IDCDT_TRACKXA2         0x0040
#define IDCDT_TRACKCDI         0x0080
#define IDCDT_MERGESESSION     0x0100
#define IDCDT_FIXDISK            0x0200

#define CDT_ALLFLAGS (IDCDT_MULTISESSION|IDCDT_FIRSTSESSION|IDCDT_USEWRITER|IDCDT_TRACKDATA|IDCDT_TRACKMODE2| \
                      IDCDT_TRACKXA1|IDCDT_TRACKXA2|IDCDT_TRACKCDI | IDCDT_MERGESESSION | IDCDT_FIXDISK)

/* Keys for CD-Author */
#define IDKEY_APPLICATION       6100
#define IDKEY_PUBLISHER         6101
#define IDKEY_PREPARER          6102
#define IDKEY_COPYRIGHT         6103
#define IDKEY_VOLUMENAME       6104

/* Private window messages */
#define WM_STARTGRABMSG "WM_STARTGRAB"
#define WM_UPDATESTATUSBARMSG "WM_UPDATESTATUSBAR"
#define WM_STARTWRITEMSG "WM_STARTWRITE"

#define NAMEFIELDLENGTH 100

/* Keys for CWMMAudio */

#define IDKEY_DISCTITLE          6200
#define IDKEY_TRACKTITLE         6201
#define IDKEY_ARTIST             6202
#define IDKEY_GENRE              6203
#define IDKEY_DISCID             6204

/* Action keys. Which helper should be started or */
/* Which helper sent the WM_APPTERMINATE msg */
#define ACKEY_IMAGEONLY        0
#define ACKEY_WRITEONLY        1
#define ACKEY_ONTHEFLY         2
#define ACKEY_PRINTSIZE         3
#define ACKEY_CDSIZE             4
#define ACKEY_WRITEAUDIO       5
#define ACKEY_PLAYTIME          6
#define ACKEY_MP3DECODE        7
#define ACKEY_SCANBUS           8


#define ACKEY_ABORT          20
#define ACKEY_MBWRITTEN          21
#define ACKEY_FIXATING          22
#define ACKEY_LEADIN          23
#define ACKEY_LEADOUT          24

/* Struct which holds data for thread */
typedef struct {
  PVOID thisPtr;
  int iWhich;
}THREADINFO;

